@@ -11,7 +11,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!KERNEL")
+(in-package "SB-KERNEL")
 
 ;;;; the CONDITION class
 
@@ -46,7 +46,7 @@
 
 (setf (condition-classoid-report (find-classoid 'condition))
       (lambda (cond stream)
-        (format stream "Condition ~/sb!impl:print-type-specifier/ was signalled."
+        (format stream "Condition ~/sb-impl:print-type-specifier/ was signalled."
                 (type-of cond))))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -73,8 +73,9 @@
     (if (and olayout
              (not (mismatch (layout-inherits olayout) new-inherits)))
         olayout
-        (make-layout :classoid (make-undefined-classoid name)
-                     :%flags +condition-layout-flag+
+        (make-layout (hash-layout-name name)
+                     (make-undefined-classoid name)
+                     :flags +condition-layout-flag+
                      :inherits new-inherits
                      :depthoid -1
                      :length (layout-length cond-layout)))))
@@ -110,24 +111,31 @@
 ;;;; slots of CONDITION objects
 
 (defun find-slot-default (class slot)
-  (let ((initargs (condition-slot-initargs slot))
-        (cpl (condition-classoid-cpl class)))
+  (multiple-value-bind (value found) (find-slot-default-initarg class slot)
     ;; When CLASS or a superclass has a default initarg for SLOT, use
     ;; that.
+    (cond (found
+           value)
+          ;; Otherwise use the initform of SLOT, if there is one.
+          ((condition-slot-initform-p slot)
+           (let ((initfun (condition-slot-initfunction slot)))
+             (aver (functionp initfun))
+             (funcall initfun)))
+          (t
+           (error "Unbound condition slot: ~S" (condition-slot-name slot))))))
+
+(defun find-slot-default-initarg (class slot)
+  (let ((initargs (condition-slot-initargs slot))
+        (cpl (condition-classoid-cpl class)))
     (dolist (class cpl)
       (let ((direct-default-initargs
               (condition-classoid-direct-default-initargs class)))
         (dolist (initarg initargs)
           (let ((initfunction (third (assoc initarg direct-default-initargs))))
             (when initfunction
-              (return-from find-slot-default (funcall initfunction)))))))
-
-    ;; Otherwise use the initform of SLOT, if there is one.
-    (if (condition-slot-initform-p slot)
-        (let ((initfun (condition-slot-initfunction slot)))
-          (aver (functionp initfun))
-          (funcall initfun))
-        (error "unbound condition slot: ~S" (condition-slot-name slot)))))
+              (return-from find-slot-default-initarg
+                (values (funcall initfunction) t)))))))
+    (values nil nil)))
 
 (defun find-condition-class-slot (condition-class slot-name)
   (dolist (sclass
@@ -147,7 +155,7 @@
       (return (setf (car (condition-slot-cell cslot)) new-value)))))
 
 (defun condition-slot-value (condition name)
-  (let ((val (getf (condition-assigned-slots condition) name sb!pcl:+slot-unbound+)))
+  (let ((val (getf (condition-assigned-slots condition) name sb-pcl:+slot-unbound+)))
     (if (unbound-marker-p val)
         (let ((class (layout-classoid (%instance-layout condition))))
           (dolist (cslot
@@ -156,13 +164,16 @@
                          (slot (or (find-condition-class-slot class name)
                                    (error "missing slot ~S of ~S" name condition))))
                      (setf (getf (condition-assigned-slots condition) name)
-                           (do ((i (+ sb!vm:instance-data-start 2) (+ i 2)))
+                           (do ((i (+ sb-vm:instance-data-start 1) (+ i 2)))
                                ((>= i instance-length) (find-slot-default class slot))
                                (when (member (%instance-ref condition i)
                                              (condition-slot-initargs slot))
                                  (return (%instance-ref condition (1+ i))))))))
             (when (eq (condition-slot-name cslot) name)
-              (return (car (condition-slot-cell cslot))))))
+              (let ((value (car (condition-slot-cell cslot))))
+                (if (unbound-marker-p value)
+                    (error "Unbound condition slot: ~S" (condition-slot-name cslot))
+                    (return value))))))
         val)))
 
 ;;;; MAKE-CONDITION
@@ -176,7 +187,7 @@
              ;; avoid direct reference to INITARGS as a list
              ;; so that it is not reified unless we reach here.
              (do-rest-arg ((arg) initargs) (push arg list))
-             (nreverse list))))
+             (list (nreverse list)))))
   ;; I am going to assume that people are not somehow getting to here
   ;; with a CLASSOID, which is not strictly legal as a designator,
   ;; but which is accepted because it is actually the desired thing.
@@ -191,14 +202,13 @@
         ;; Interestingly we fail to validate the actual-initargs,
         ;; allowing any random initarg names.  Is this permissible?
         ;; And why is lazily filling in ASSIGNED-SLOTS beneficial anyway?
-        (let ((instance (%make-instance (+ sb!vm:instance-data-start
-                                           2 ; ASSIGNED-SLOTS and HASH
+        (let ((instance (%make-instance (+ sb-vm:instance-data-start
+                                           1 ; ASSIGNED-SLOTS
                                            (length initargs))))) ; rest
           (setf (%instance-layout instance) (classoid-layout classoid)
-                (condition-assigned-slots instance) nil
-                (condition-hash instance) (sb!impl::new-instance-hash-code))
+                (condition-assigned-slots instance) nil)
           (do-rest-arg ((val index) initargs)
-            (setf (%instance-ref instance (+ sb!vm:instance-data-start index 2)) val))
+            (setf (%instance-ref instance (+ sb-vm:instance-data-start index 1)) val))
           (values instance classoid))
         (error 'simple-type-error
                :datum designator
@@ -221,15 +231,20 @@
 
     ;; Set any class slots with initargs present in this call.
     (dolist (cslot (condition-classoid-class-slots classoid))
-      (dolist (initarg (condition-slot-initargs cslot))
-        (let ((val (getf initargs initarg sb!pcl:+slot-unbound+)))
-          (unless (unbound-marker-p val)
-            (setf (car (condition-slot-cell cslot)) val)))))
+      (loop for (key value) on initargs by #'cddr
+            when (memq key (condition-slot-initargs cslot))
+            do (setf (car (condition-slot-cell cslot)) value)
+               (return)
+            finally
+            (multiple-value-bind (value found)
+                (find-slot-default-initarg classoid cslot)
+              (when found
+                (setf (car (condition-slot-cell cslot)) value)))))
 
     ;; Default any slots with non-constant defaults now.
     (dolist (hslot (condition-classoid-hairy-slots classoid))
       (when (dolist (initarg (condition-slot-initargs hslot) t)
-              (unless (unbound-marker-p (getf initargs initarg sb!pcl:+slot-unbound+))
+              (unless (unbound-marker-p (getf initargs initarg sb-pcl:+slot-unbound+))
                 (return nil)))
         (setf (getf (condition-assigned-slots condition)
                     (condition-slot-name hslot))
@@ -296,7 +311,7 @@
          t
          `(condition-slot-writer ,name))))
 
-(!defvar *define-condition-hooks* nil)
+(!define-load-time-global *define-condition-hooks* nil)
 
 (defun %set-condition-report (name report)
   (setf (condition-classoid-report (find-classoid name))
@@ -309,9 +324,9 @@
    'condition name
    (lambda ()
      (%%compiler-define-condition name parent-types layout all-readers all-writers)
-     (when source-location
-       (setf (layout-source-location layout) source-location))
      (let ((classoid (find-classoid name)))
+       (when source-location
+         (setf (classoid-source-location classoid) source-location))
        (setf (condition-classoid-slots classoid) slots
              (condition-classoid-direct-default-initargs classoid) direct-default-initargs
              (documentation name 'type) documentation)
@@ -342,7 +357,7 @@
                                 (let ((initfun (condition-slot-initfunction slot)))
                                   (aver (functionp initfun))
                                   (funcall initfun))
-                                sb!pcl:+slot-unbound+))))
+                                sb-pcl:+slot-unbound+))))
               (push slot (condition-classoid-class-slots classoid)))
              ((:instance nil)
               (setf (condition-slot-allocation slot) :instance)
@@ -375,6 +390,7 @@
    Condition types are classes, but (as allowed by ANSI and not as described in
    CLtL2) are neither STANDARD-OBJECTs nor STRUCTURE-OBJECTs. WITH-SLOTS and
    SLOT-VALUE may not be used on condition objects."
+  (check-designator name define-condition)
   (let* ((parent-types (or parent-types '(condition)))
          (layout (find-condition-layout name parent-types))
          (documentation nil)
@@ -384,61 +400,64 @@
               (all-readers nil append)
               (all-writers nil append))
       (dolist (spec slot-specs)
-        (when (keywordp spec)
-          (warn "Keyword slot name indicates probable syntax error:~%  ~S"
-                spec))
-        (let* ((spec (if (consp spec) spec (list spec)))
-               (slot-name (first spec))
-               (allocation :instance)
-               (initform-p nil)
-               documentation
-               initform)
-          (collect ((initargs)
-                    (readers)
-                    (writers))
-            (do ((options (rest spec) (cddr options)))
-                ((null options))
-              (unless (and (consp options) (consp (cdr options)))
-                (error "malformed condition slot spec:~%  ~S." spec))
-              (let ((arg (second options)))
-                (case (first options)
-                  (:reader (readers arg))
-                  (:writer (writers arg))
-                  (:accessor
-                   (readers arg)
-                   (writers `(setf ,arg)))
-                  (:initform
-                   (when initform-p
-                     (error "more than one :INITFORM in ~S" spec))
-                   (setq initform-p t)
-                   (setq initform arg))
-                  (:initarg (initargs arg))
-                  (:allocation
-                   (setq allocation arg))
-                  (:documentation
-                   (when documentation
-                     (error "more than one :DOCUMENTATION in ~S" spec))
-                   (unless (stringp arg)
-                     (error "slot :DOCUMENTATION argument is not a string: ~S"
-                            arg))
-                   (setq documentation arg))
-                  (:type)
-                  (t
-                   (error "unknown slot option:~%  ~S" (first options))))))
+        (with-current-source-form (spec)
+          (when (keywordp spec)
+            (warn "Keyword slot name indicates probable syntax error:~%  ~S"
+                  spec))
+          (let* ((spec (if (consp spec) spec (list spec)))
+                 (slot-name (first spec))
+                 (allocation :instance)
+                 (initform-p nil)
+                 documentation
+                 initform)
+            (collect ((initargs)
+                      (readers)
+                      (writers))
+              (do ((options (rest spec) (cddr options)))
+                  ((null options))
+                (unless (and (consp options) (consp (cdr options)))
+                  (error "malformed condition slot spec:~%  ~S." spec))
+                (let ((arg (second options)))
+                  (case (first options)
+                    (:reader (readers arg))
+                    (:writer (writers arg))
+                    (:accessor
+                     (readers arg)
+                     (writers `(setf ,arg)))
+                    (:initform
+                     (when initform-p
+                       (error "more than one :INITFORM in ~S" spec))
+                     (setq initform-p t)
+                     (setq initform arg))
+                    (:initarg (initargs arg))
+                    (:allocation
+                     (setq allocation arg))
+                    (:documentation
+                     (when documentation
+                       (error "more than one :DOCUMENTATION in ~S" spec))
+                     (unless (stringp arg)
+                       (error "slot :DOCUMENTATION argument is not a string: ~S"
+                              arg))
+                     (setq documentation arg))
+                    (:type
+                     (check-slot-type-specifier
+                      arg slot-name (cons 'define-condition name)))
+                    (t
+                     (error "unknown slot option:~%  ~S" (first options))))))
 
-            (all-readers (readers))
-            (all-writers (writers))
-            (slots `(make-condition-slot
-                     :name ',slot-name
-                     :initargs ',(initargs)
-                     :readers ',(readers)
-                     :writers ',(writers)
-                     :initform-p ',initform-p
-                     :documentation ',documentation
-                     :initform ,(when initform-p `',initform)
-                     :initfunction ,(when initform-p
-                                      `#'(lambda () ,initform))
-                     :allocation ',allocation)))))
+              (all-readers (readers))
+              (all-writers (writers))
+              (slots `(make-condition-slot
+                       :name ',slot-name
+                       :initargs ',(initargs)
+                       :readers ',(readers)
+                       :writers ',(writers)
+                       :initform-p ',initform-p
+                       :documentation ',documentation
+                       :initform ,(when initform-p `',initform)
+                       :initfunction ,(when initform-p
+                                        `#'(lambda () ,initform))
+                       :allocation ',allocation))))))
 
       (dolist (option options)
         (unless (consp option)
@@ -463,19 +482,23 @@
            (error "unknown option: ~S" (first option)))))
 
       ;; Maybe kill docstring, but only under the cross-compiler.
-      #!+(and (not sb-doc) (host-feature sb-xc-host)) (setq documentation nil)
+      #+(and (not sb-doc) sb-xc-host) (setq documentation nil)
       `(progn
-         (eval-when (:compile-toplevel)
-           (%compiler-define-condition ',name ',parent-types ',layout
-                                       ',(all-readers) ',(all-writers)))
+         ,@(when *top-level-form-p*
+             ;; Avoid dumping uninitialized layouts, for sb-fasl::dump-layout
+             `((eval-when (:compile-toplevel)
+                 (%compiler-define-condition ',name ',parent-types ,layout
+                                             ',(all-readers) ',(all-writers)))))
          (%define-condition ',name
                             ',parent-types
-                            ',layout
+                            ,(if *top-level-form-p*
+                                 layout
+                                 `(find-condition-layout ',name ',parent-types))
                             (list ,@(slots))
                             (list ,@direct-default-initargs)
                             ',(all-readers)
                             ',(all-writers)
-                            (sb!c:source-location)
+                            (sb-c:source-location)
                             ,@(and documentation
                                    `(,documentation)))
          ;; This needs to be after %DEFINE-CONDITION in case :REPORT
@@ -549,7 +572,7 @@
        (format stream  "~@<The value ~
                       ~@:_~2@T~S ~
                       ~@:_is not of type ~
-                      ~@:_~2@T~/sb!impl:print-type-specifier/~@[ ~
+                      ~@:_~2@T~/sb-impl:print-type-specifier/~@[ ~
                       ~@:_~a~]~:@>"
                (type-error-datum condition)
                type
@@ -578,6 +601,11 @@
    (lambda (condition stream)
      (format stream "~S is closed" (stream-error-stream condition)))))
 
+(define-condition closed-saved-stream-error (closed-stream-error) ()
+  (:report
+   (lambda (condition stream)
+     (format stream "~S was closed by SB-EXT:SAVE-LISP-AND-DIE" (stream-error-stream condition)))))
+
 (define-condition file-error (error)
   ((pathname :reader file-error-pathname :initarg :pathname))
   (:report
@@ -619,16 +647,20 @@
   ((not-yet-loaded :initform nil :reader not-yet-loaded :initarg :not-yet-loaded))
   (:report
    (lambda (condition stream)
-     (let ((*package* (find-package :keyword)))
+     (let ((name (cell-error-name condition)))
        (format stream
-               "~@<The function ~S is undefined.~@?~@:>"
-               (cell-error-name condition)
+               (if (and (symbolp name) (macro-function name))
+                   (sb-format:tokens "~@<~/sb-ext:print-symbol-with-prefix/ is a macro, ~
+                                      not a function.~@:>")
+                   (sb-format:tokens "~@<The function ~/sb-ext:print-symbol-with-prefix/ ~
+                                      is undefined.~@?~@:>"))
+               name
                (case (not-yet-loaded condition)
                  (:local
-                  "~:@_It is a local function ~
-                       not available at compile-time.")
-                 ((t) "~:@_It is defined earlier in the ~
-                           file but is not available at compile-time.")
+                  (sb-format:tokens "~:@_It is a local function ~
+                                     not available at compile-time."))
+                 ((t) (sb-format:tokens "~:@_It is defined earlier in the ~
+                                         file but is not available at compile-time."))
                  (t
                   "")))))))
 
@@ -744,6 +776,11 @@
 (define-condition simple-storage-condition (storage-condition simple-condition)
   ())
 
+(define-condition sanitizer-error (simple-error)
+  ((value :reader sanitizer-error-value :initarg :value)
+   (address :reader sanitizer-error-address :initarg :address)
+   (size :reader sanitizer-error-size :initarg :size)))
+
 ;;; a condition for use in stubs for operations which aren't supported
 ;;; on some platforms
 ;;;
@@ -892,7 +929,7 @@
   ()
   (:default-initargs
       :references '((:ansi-cl :function make-array)
-                    (:ansi-cl :function sb!xc:upgraded-array-element-type))))
+                    (:ansi-cl :function sb-xc:upgraded-array-element-type))))
 
 (define-condition type-warning (reference-condition simple-warning)
   ()
@@ -900,6 +937,7 @@
 (define-condition type-style-warning (reference-condition simple-style-warning)
   ()
   (:default-initargs :references '((:sbcl :node "Handling of Types"))))
+(define-condition slot-initform-type-style-warning (type-style-warning) ())
 
 (define-condition local-argument-mismatch (reference-condition simple-warning)
   ()
@@ -1055,7 +1093,7 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
      (let ((*print-array* nil))
        (format stream
                "~@<Displaced array originally of type ~
-                 ~/sb!impl:print-type-specifier/ has been invalidated ~
+                 ~/sb-impl:print-type-specifier/ has been invalidated ~
                  due its displaced-to array ~S having become too small ~
                  to hold it: the displaced array's dimensions have all ~
                  been set to zero to trap accesses to it.~:@>"
@@ -1066,12 +1104,26 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
       (list '(:ansi-cl :function adjust-array))))
 
 (define-condition index-too-large-error (type-error)
-  ()
+  ((sequence :initarg :sequence))
   (:report
    (lambda (condition stream)
-     (format stream
-             "The index ~S is too large."
-             (type-error-datum condition)))))
+     (let ((sequence (slot-value condition 'sequence))
+           (index (type-error-datum condition)))
+       (if (vectorp sequence)
+           (format stream "Invalid index ~W for ~S ~@[with fill-pointer ~a~], ~
+                           should be a non-negative integer below ~W."
+                   index
+                   (type-of sequence)
+                   (and (array-has-fill-pointer-p sequence)
+                        (fill-pointer sequence))
+                   (length sequence))
+           (format stream
+                   "The index ~S is too large for a ~a of length ~s."
+                   index
+                   (if (listp sequence)
+                       "list"
+                       "sequence")
+                   (length sequence)))))))
 
 (define-condition bounding-indices-bad-error (reference-condition type-error)
   ((object :reader bounding-indices-bad-object :initarg :object))
@@ -1106,7 +1158,7 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
                      "An attempt to access an array of element-type ~
                       NIL was made.  Congratulations!")))
   (:default-initargs
-      :references '((:ansi-cl :function sb!xc:upgraded-array-element-type)
+      :references '((:ansi-cl :function sb-xc:upgraded-array-element-type)
                     (:ansi-cl :section (15 1 2 1))
                     (:ansi-cl :section (15 1 2 2)))))
 
@@ -1160,7 +1212,10 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
 
 (define-condition simple-package-error (simple-condition package-error) ())
 
+(define-condition package-does-not-exist (simple-package-error) ())
+
 (define-condition simple-reader-package-error (simple-reader-error package-error) ())
+(define-condition reader-package-does-not-exist (simple-reader-package-error package-does-not-exist) ())
 
 (define-condition reader-eof-error (end-of-file)
   ((context :reader reader-eof-error-context :initarg :context))
@@ -1178,7 +1233,7 @@ SB-EXT:PACKAGE-LOCKED-ERROR-SYMBOL."))
      (let ((error-stream (stream-error-stream condition)))
        (format stream
                "READER-ERROR ~@[at ~W ~]on ~S:~%~?~%Original error: ~A"
-               (sb!impl::file-position-or-nil-for-error error-stream) error-stream
+               (sb-impl::file-position-or-nil-for-error error-stream) error-stream
                (simple-condition-format-control condition)
                (simple-condition-format-arguments condition)
                (reader-impossible-number-error-error condition))))))
@@ -1233,9 +1288,10 @@ longer than permitted by the deadline."))
                                                    simple-error)
   ()
   (:default-initargs
-   :format-control  "Symbol ~/sb-ext:print-symbol-with-prefix/ cannot ~
-                     be both the name of a type and the name of a ~
-                     declaration"
+   :format-control
+   #.(sb-xc:macroexpand-1 ; stuff in a literal #<fmt-control>
+      '(sb-format:tokens "Symbol ~/sb-ext:print-symbol-with-prefix/ cannot ~
+     be both the name of a type and the name of a declaration"))
    :references '((:ansi-cl :section (3 8 21)))))
 
 ;;; Single stepping conditions
@@ -1354,23 +1410,21 @@ handled by any other handler, it will be muffled.")
 ;;;; Deciding which redefinitions are "interesting".
 
 (defun function-file-namestring (function)
-  #!+sb-eval
-  (when (typep function 'sb!eval:interpreted-function)
+  (when (typep function 'interpreted-function)
     (return-from function-file-namestring
-      (sb!c:definition-source-location-namestring
-          (sb!eval:interpreted-function-source-location function))))
-  #!+sb-fasteval
-  (when (typep function 'sb!interpreter:interpreted-function)
-    (return-from function-file-namestring
-      (awhen (sb!interpreter:fun-source-location function)
-        (sb!c:definition-source-location-namestring it))))
+      #+sb-eval
+      (sb-c:definition-source-location-namestring
+          (sb-eval:interpreted-function-source-location function))
+      #+sb-fasteval
+      (awhen (sb-interpreter:fun-source-location function)
+        (sb-c:definition-source-location-namestring it))))
   (let* ((fun (%fun-fun function))
          (code (fun-code-header fun))
          (debug-info (%code-debug-info code))
          (debug-source (when debug-info
-                         (sb!c::debug-info-source debug-info)))
+                         (sb-c::debug-info-source debug-info)))
          (namestring (when debug-source
-                       (sb!c::debug-source-namestring debug-source))))
+                       (debug-source-namestring debug-source))))
     namestring))
 
 (defun interesting-function-redefinition-warning-p (warning old)
@@ -1381,9 +1435,7 @@ handled by any other handler, it will be muffled.")
           (typep new '(not compiled-function)))
      ;; fin->regular is interesting except for interpreted->compiled.
      (and (typep new '(not funcallable-instance))
-          (typep old '(and funcallable-instance
-                       #!+sb-fasteval (not sb!interpreter:interpreted-function)
-                       #!+sb-eval (not sb!eval:interpreted-function))))
+          (typep old '(and funcallable-instance (not interpreted-function))))
      ;; different file or unknown location is interesting.
      (let* ((old-namestring (function-file-namestring old))
             (new-namestring (function-file-namestring new)))
@@ -1421,12 +1473,12 @@ handled by any other handler, it will be muffled.")
    (let* ((name (redefinition-warning-name warning))
           (old (fdefinition name))
           (old-location (when (typep old 'generic-function)
-                          (sb!pcl::definition-source old)))
+                          (sb-pcl::definition-source old)))
           (old-namestring (when old-location
-                            (sb!c:definition-source-location-namestring old-location)))
+                            (sb-c:definition-source-location-namestring old-location)))
           (new-location (redefinition-warning-new-location warning))
           (new-namestring (when new-location
-                           (sb!c:definition-source-location-namestring new-location))))
+                           (sb-c:definition-source-location-namestring new-location))))
      (and old-namestring
           new-namestring
           (string= old-namestring new-namestring)))))
@@ -1439,12 +1491,12 @@ handled by any other handler, it will be muffled.")
    ;; Can't use the shared logic above, since GF's don't get a "new"
    ;; definition -- rather the FIN-FUNCTION is set.
    (let* ((old-method (redefinition-with-defmethod-old-method warning))
-          (old-location (sb!pcl::definition-source old-method))
+          (old-location (sb-pcl::definition-source old-method))
           (old-namestring (when old-location
-                            (sb!c:definition-source-location-namestring old-location)))
+                            (sb-c:definition-source-location-namestring old-location)))
           (new-location (redefinition-warning-new-location warning))
           (new-namestring (when new-location
-                            (sb!c:definition-source-location-namestring new-location))))
+                            (sb-c:definition-source-location-namestring new-location))))
          (and new-namestring
               old-namestring
               (string= new-namestring old-namestring)))))
@@ -1480,6 +1532,10 @@ the usual naming convention (names like *FOO*) for special variables"
     (dubious-asterisks-around-variable-name)
   ())
 
+(define-condition &optional-and-&key-in-lambda-list
+    (style-warning simple-condition)
+  ())
+
 ;; We call this UNDEFINED-ALIEN-STYLE-WARNING because there are some
 ;; subclasses of ERROR above having to do with undefined aliens.
 (define-condition undefined-alien-style-warning (style-warning)
@@ -1488,7 +1544,9 @@ the usual naming convention (names like *FOO*) for special variables"
              (format stream "Undefined alien: ~S"
                      (undefined-alien-symbol warning)))))
 
-#!+(or sb-eval sb-fasteval)
+;;; Formerly this was guarded by "#+(or sb-eval sb-fasteval)", but
+;;; why would someone build with no interpreter? And if they did,
+;;; would they really care that one extra condition definition exists?
 (define-condition lexical-environment-too-complex (style-warning)
   ((form :initarg :form :reader lexical-environment-too-complex-form)
    (lexenv :initarg :lexenv :reader lexical-environment-too-complex-lexenv))
@@ -1566,10 +1624,10 @@ the usual naming convention (names like *FOO*) for special variables"
    (lambda (condition stream)
      (format stream
              "~@<The new ~A proclamation for~@[ ~A~] ~
-               ~/sb!impl:print-symbol-with-prefix/~
-               ~@:_~2@T~/sb!impl:print-type-specifier/~@:_~
+               ~/sb-ext:print-symbol-with-prefix/~
+               ~@:_~2@T~/sb-impl:print-type-specifier/~@:_~
                does not match the old ~4:*~A~3* proclamation~
-               ~@:_~2@T~/sb!impl:print-type-specifier/~@:>"
+               ~@:_~2@T~/sb-impl:print-type-specifier/~@:>"
              (proclamation-mismatch-kind condition)
              (proclamation-mismatch-description condition)
              (proclamation-mismatch-name condition)
@@ -1656,9 +1714,7 @@ conditions."))
                             (deprecation-condition-software condition)
                             (deprecation-condition-name condition)))))))
 
-  ;; PRINT-SYMBOL-WITH-PREFIX is spelled using its target package name,
-  ;; not its cold package name, because these methods aren't unsable until
-  ;; warm load. (!CALL-A-METHOD does not understand method qualifiers)
+  ;; These conditions must not occur in self-build!
   (define-deprecation-warning early-deprecation-warning style-warning nil
      "~%~@<~:@_In future~@[ ~A~] versions ~
       ~/sb-ext:print-symbol-with-prefix/ will signal a full warning ~
@@ -1745,11 +1801,11 @@ CONTROL-ERROR if the restart does not exist.")
   (def step-next
       "Transfers control to the STEP-NEXT restart associated with the
 condition, executing the current form without stepping and continuing
-stepping with the next form. Signals CONTROL-ERROR is the restart does
-not exists.")
+stepping with the next form. Signals CONTROL-ERROR if the restart does
+not exist.")
   (def step-into
       "Transfers control to the STEP-INTO restart associated with the
-condition, stepping into the current form. Signals a CONTROL-ERROR is
+condition, stepping into the current form. Signals a CONTROL-ERROR if
 the restart does not exist."))
 
 ;;; Compiler macro magic
@@ -1764,7 +1820,7 @@ the restart does not exist."))
 
 ;; After (or if) we deem this the optimal name for this condition,
 ;; it should be exported from SB-EXT so that people can muffle it.
-(define-condition sb!c:inlining-dependency-failure (simple-style-warning) ())
+(define-condition sb-c:inlining-dependency-failure (simple-style-warning) ())
 
 
 (define-condition layout-invalid (type-error)
@@ -1800,21 +1856,24 @@ the restart does not exist."))
                        (program-error-source condition)
                        (program-error-message condition)))))
 
-(define-condition interpreted-program-error
-    (program-error encapsulated-condition)
-  ;; Unlike COMPILED-PROGRAM-ERROR, we don't need to dump these, so
-  ;; storing the original condition and form is OK.
-  ((form :initarg :form :reader program-error-form))
-  (:report (lambda (condition stream)
-             (format stream "~&Evaluation of~%  ~S~%~
-                             caused error:~%  ~A~%"
-                     (program-error-form condition)
-                     (encapsulated-condition condition)))))
-
 (define-condition simple-control-error (simple-condition control-error) ())
 (define-condition simple-file-error    (simple-condition file-error)    ())
-(define-condition simple-stream-error  (simple-condition stream-error)  ())
-(define-condition simple-parse-error   (simple-condition parse-error)   ())
+
+(defun %file-error (pathname &optional datum &rest arguments)
+  (typecase datum
+    (format-control (error 'simple-file-error :pathname pathname
+                                              :format-control datum
+                                              :format-arguments arguments))
+    (t (apply #'error datum :pathname pathname arguments))))
+
+(define-condition file-exists (simple-file-error) ())
+(define-condition file-does-not-exist (simple-file-error) ())
+(define-condition delete-file-error (simple-file-error) ())
+
+(define-condition simple-stream-error (simple-condition stream-error) ())
+(define-condition simple-parse-error  (simple-condition parse-error)  ())
+
+(define-condition broken-pipe (simple-stream-error) ())
 
 (define-condition character-coding-error (error)
   ((external-format :initarg :external-format :reader character-coding-error-external-format)))
