@@ -140,25 +140,28 @@
         ;; provided that SB-XC:CHAR-CODE doesn't fail, the answer is certain
         (values (test-character-type type) t))
        (classoid ; = {built-in,structure,condition,standard,static}-classoid
-        (if (built-in-classoid-p type)
-            (ecase (classoid-name type)
-              (symbol (values (symbolp obj) t)) ; 1:1 correspondence with host
-              (function
-               (if (functionp obj)
-                   (uncertain)
-                   ;; probably not a function. What about FMT-CONTROL instances?
-                   (values nil t)))
-              ((system-area-pointer stream fdefn weak-pointer file-stream
-                code-component lra)
-               (values nil t)))
-            (if (not (%instancep obj))
-                (values nil t) ; false certainly
-                (let ((name (classoid-name type)))
-                  (if (and (cl:find-class name nil) ; see if the host knows the type
-                           ;; and it's in our object hierarchy
-                           (cl:subtypep name 'structure!object))
-                      (values (cl:typep obj name) t)
-                      (unimplemented))))))
+        (let ((name (classoid-name type)))
+          (if (built-in-classoid-p type)
+              (ecase name
+                (symbol (values (symbolp obj) t)) ; 1:1 correspondence with host
+                (function
+                 (if (functionp obj)
+                     (uncertain)
+                     ;; probably not a function. What about FMT-CONTROL instances?
+                     (values nil t)))
+                ((system-area-pointer stream fdefn weak-pointer file-stream
+                  code-component lra)
+                 (values nil t)))
+              (cond ((eq name 'pathname)
+                     (values (pathnamep obj) t))
+                    ((not (%instancep obj))
+                     (values nil t)) ; false certainly
+                    (t
+                     (if (and (cl:find-class name nil) ; see if the host knows the type
+                              ;; and it's in our object hierarchy
+                              (cl:subtypep name 'structure!object))
+                         (values (cl:typep obj name) t)
+                       (unimplemented)))))))
        (fun-type
         (if (fun-designator-type-p type)
              (values (typep obj '(or symbol function)) t)
@@ -186,6 +189,14 @@
                       ;; See if the host knows our DEF!STRUCT yet
                       (cl:subtypep spec 'structure!object))
                  (values (cl:typep obj spec) t))
+                ;; Sometimes we try to test a forward-referenced type
+                ;; that was unknown at the point of creation but has
+                ;; been resolved at the point of testing via block
+                ;; compilation. Retry with the now resolved type if
+                ;; this is the case.
+                ((and (symbolp spec)
+                      (not (unknown-type-p (specifier-type spec))))
+                 (cross-typep caller obj (specifier-type spec)))
                 ;; Special-case a few forward-referenced instance types.
                 ;; Some are picked off by the quick exit case in (DEFUN SB-XC:TYPEP)
                 ;; but a bunch of other cases arise within the type methods per se
@@ -217,7 +228,7 @@
                              condition restart
                              synonym-stream
                              ;; why on earth is LABEL needed here?
-                             sb-assem::label))))
+                             sb-assem:label))))
                  #+nil (format t "~&(XTYPEP '~S '~S) -> (NIL T)~%" obj spec)
                  (values nil t))
                 (t
@@ -234,7 +245,11 @@
                       (t
                        (uncertain))))
               (unimplemented)))))
-    (when (and (not certain) *xtypep-uncertainty-action*)
+    (when (and (not certain) *xtypep-uncertainty-action*
+               ;; KLUDGE: Allow some slack while block compiling, as we are
+               ;; hoping that forward referenced types get resolved.
+               (not (and (boundp 'sb-c::*compilation*)
+                         (eq (sb-c::block-compile sb-c::*compilation*) t))))
       ;; can't even backtrace if the printing of a something involving
       ;; uncertainty involves uncertainty.
       (let* ((action *xtypep-uncertainty-action*)
@@ -288,7 +303,10 @@
                                      (typep (hairy-type-specifier type)
                                             '(cons (eql satisfies))))
                             (return-from contains-satisfies t)))
-                        ctype)))
+                        ctype))
+            ;; KLUDGE: Allow uncertainty while block compiling.
+            (and (boundp 'sb-c::*compilation*)
+                 (eq (sb-c::block-compile sb-c::*compilation*) t)))
         (values answer certain)
         (warn 'cross-type-giving-up :call `(ctypep ,obj ,ctype)))))
 
