@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!FASL")
+(in-package "SB-FASL")
 
 ;;;; various constants and essentially-constants
 
@@ -33,23 +33,21 @@
 ;;;   against.
 (defglobal *fasl-header-string-start-string* "# FASL")
 
-;;; a list of SB!XC:*FEATURES* flags which affect binary compatibility,
+;;; a list of SB-XC:*FEATURES* flags which affect binary compatibility,
 ;;; i.e. which must be the same between the SBCL which compiled the code
 ;;; and the SBCL which executes the code. This is a property of SBCL executables
 ;;; in the abstract, not of this particular SBCL executable,
 ;;; so any flag in this list may or may not be present
 ;;; in the *FEATURES* list of this particular build.
 (defglobal *features-potentially-affecting-fasl-format*
-    (append '(:sb-thread :sb-package-locks :sb-unicode :cheneygc
-              :gencgc :msan :sb-safepoint :sb-safepoint-strictly
-              :sb-dynamic-core)
-            #!+(or x86 x86-64) '(:int4-breakpoints :ud2-breakpoints)))
+    (append '(:sb-thread :sb-unicode :cheneygc
+              :gencgc :msan :sb-safepoint :sb-safepoint-strictly)))
 
 ;;; Return a string representing symbols in *FEATURES-POTENTIALLY-AFFECTING-FASL-FORMAT*
 ;;; which are present in a particular compilation.
 (defun compute-features-affecting-fasl-format ()
   (let ((list (sort (copy-list (intersection *features-potentially-affecting-fasl-format*
-                                             sb!xc:*features*))
+                                             sb-xc:*features*))
                     #'string< :key #'symbol-name)))
     ;; Stringify the subset of *FEATURES* that affect fasl format.
     ;; A list would be the natural representation choice for this, but a string
@@ -131,21 +129,26 @@
 
 ;;;; the FOP database
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  ;; The bottom 5 bits of the opcodes above 128 encode an implicit operand.
+  (defconstant n-ordinary-fops 128))
+
 ;;; a vector indexed by a FaslOP that yields a function which performs
 ;;; the operation. Most functions take 0 arguments - they only manipulate
 ;;; the fop stack. But if the fop is defined to receive an argument (or two)
 ;;; then loader's main loop is responsible for supplying it.
-(defglobal **fop-funs** (make-array 256 :initial-element 0))
-(declaim (simple-vector **fop-funs**))
+(defglobal **fop-funs** (make-array n-ordinary-fops :initial-element 0))
+(declaim (type (simple-vector #.n-ordinary-fops) **fop-funs**))
 
 ;;; Two arrays indicate fop function signature.
 ;;; The first array indicates how many integer operands follow the opcode.
 ;;; The second tells whether the fop wants its result pushed on the stack.
-(declaim (type (cons (simple-array (mod 4) (256)) (simple-bit-vector 256))
+(declaim (type (cons (simple-array (mod 4) (#.n-ordinary-fops))
+                     (simple-bit-vector #.n-ordinary-fops))
                **fop-signatures**))
 (defglobal **fop-signatures**
-    (cons (make-array 256 :element-type '(mod 4) :initial-element 0)
-          (make-array 256 :element-type 'bit :initial-element 0)))
+    (cons (make-array n-ordinary-fops :element-type '(mod 4) :initial-element 0)
+          (make-array n-ordinary-fops :element-type 'bit :initial-element 0)))
 
 ;;;; variables
 
@@ -174,7 +177,7 @@
   ;; function calls) while executing other FOPs. SKIP-UNTIL will
   ;; either contain the position where the skipping will stop, or
   ;; NIL if we're executing normally.
-  (skip-until nil))
+  (skip-until nil :type (or null fixnum)))
 (declaim (freeze-type fasl-input))
 
 ;;; Unique number assigned into high 4 bytes of 64-bit code size slot
