@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; Limitation: no list might have more than INDEX conses.
 
@@ -18,7 +18,7 @@
 ;;;; -- WHN 20000127
 
 (declaim (maybe-inline
-          tree-equal %setnth nthcdr
+          tree-equal %setnth nthcdr nth
           tailp union
           nunion intersection nintersection set-difference nset-difference
           set-exclusive-or nset-exclusive-or subsetp acons
@@ -143,7 +143,8 @@
                       (y y (cdr y)))
                      ((or (atom x)
                           (atom y))
-                      (eql x y))
+                      (or (eql x y)
+                          (return-from tree-equal-eql)))
                    (cond ((consp (car x))
                           (when (atom (car y))
                             (return-from tree-equal-eql))
@@ -187,8 +188,23 @@
 
 (defun nth (n list)
   "Return the nth object in a list where the car is the zero-th element."
-  (declare (explicit-check))
-  (car (nthcdr n list)))
+  (declare (explicit-check)
+           (optimize speed))
+  (typecase n
+    ((and fixnum unsigned-byte)
+     (block nil
+       (let ((i n)
+             (result list))
+         (tagbody
+          loop
+            (the list result)
+            (if (plusp i)
+                (psetq i (1- i)
+                       result (cdr result))
+                (return (car result)))
+            (go loop)))))
+    (t
+     (car (nthcdr n list)))))
 
 (defun first (list)
   "Return the 1st object in a list or NIL if the list is empty."
@@ -226,41 +242,34 @@
 
 (defun nthcdr (n list)
   "Performs the cdr function n times on a list."
+  (declare (explicit-check n)
+           (optimize speed))
   (flet ((fast-nthcdr (n list)
-           (declare (type index n))
            (do ((i n (1- i))
                 (result list (cdr result)))
-               ((not (plusp i)) result)
-             (declare (type index i)))))
+               ((not (plusp i)) result))))
     (typecase n
-      (index (fast-nthcdr n list))
+      ((and fixnum unsigned-byte)
+       (fast-nthcdr n list))
       ;; Such a large list can only be circular
-      (t (do ((i 0 (1+ i))
+      (t
+       (locally (declare (unsigned-byte n))
+         (do ((i 0 (1+ i))
               (r-i list (cdr r-i))
               (r-2i list (cddr r-2i)))
              ((and (eq r-i r-2i) (not (zerop i)))
               (fast-nthcdr (mod n i) r-i))
-           (declare (type index i)))))))
+           (declare (type fixnum i))))))))
 
 ;;; For [n]butlast
 (defun dotted-nthcdr (n list)
-  (flet ((fast-nthcdr (n list)
-           (declare (type index n))
-           (do ((i n (1- i))
-                (result list (cdr result)))
-               ((not (plusp i)) result)
-             (declare (type index i))
-             (when (atom result)
-               (return)))))
-    (typecase n
-      (index (fast-nthcdr n list))
-      ;; Such a large list can only be circular
-      (t (do ((i 0 (1+ i))
-              (r-i list (cdr r-i))
-              (r-2i list (cddr r-2i)))
-             ((and (eq r-i r-2i) (not (zerop i)))
-              (fast-nthcdr (mod n i) r-i))
-           (declare (type index i)))))))
+  (declare (fixnum n))
+  (do ((i n (1- i))
+       (result list (cdr result)))
+      ((not (plusp i)) result)
+    (declare (type fixnum i))
+    (when (atom result)
+      (return))))
 
 ;;; LAST
 ;;;
@@ -301,15 +310,15 @@
                 returned-list)))
 
   (defun %last0 (list)
-    (declare (optimize speed (sb!c::verify-arg-count 0)))
+    (declare (optimize speed (sb-c:verify-arg-count 0)))
     (last0-macro))
 
   (defun %last1 (list)
-    (declare (optimize speed (sb!c::verify-arg-count 0)))
+    (declare (optimize speed (sb-c:verify-arg-count 0)))
     (last1-macro))
 
   (defun %lastn/fixnum (list n)
-    (declare (optimize speed (sb!c::verify-arg-count 0))
+    (declare (optimize speed (sb-c:verify-arg-count 0))
              (type (and unsigned-byte fixnum) n))
     (case n
       (1 (last1-macro))
@@ -317,7 +326,7 @@
       (t (lastn-macro fixnum))))
 
   (defun %lastn/bignum (list n)
-    (declare (optimize speed (sb!c::verify-arg-count 0))
+    (declare (optimize speed (sb-c:verify-arg-count 0))
              (type (and unsigned-byte bignum) n))
     (lastn-macro unsigned-byte))
 
@@ -334,7 +343,7 @@
           (lastn-macro unsigned-byte)))))))
 
 (define-compiler-macro last (&whole form list &optional (n 1) &environment env)
-  (if (sb!xc:constantp n env)
+  (if (sb-xc:constantp n env)
       (case (constant-form-value n env)
         (0 `(%last0 ,list))
         (1 `(%last1 ,list))
@@ -410,7 +419,7 @@
        (cdr result)))))
 
 (defun append2 (x y)
-  (declare (optimize (sb!c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (if (null x)
       y
       (let ((result (list (car x))))
@@ -424,21 +433,6 @@
 
 ;;;; list copying functions
 
-(defmacro copy-list-macro (list &key check-proper-list)
-    ;; Unless CHECK-PROPER-LIST is true, the list is copied correctly
-    ;; even if the list is not terminated by NIL. The new list is built
-    ;; by CDR'ing SPLICE which is always at the tail of the new list.
-    `(when ,list
-       (let ((copy (list (car ,list))))
-         (do ((orig (cdr ,list) (cdr orig))
-              (splice copy (cdr (rplacd splice (cons (car orig) nil)))))
-             (,@(if check-proper-list
-                    '((endp orig))
-                    '((atom orig)
-                      (unless (null orig)
-                        (rplacd splice orig))))
-              copy)))))
-
 (defun copy-list (list)
   "Return a new list which is EQUAL to LIST. LIST may be improper."
   (copy-list-macro list))
@@ -537,14 +531,17 @@
 
 
 (defun butlast (list &optional (n 1))
-  (cond ((zerop n)
-         (copy-list list))
-        ((not (typep n 'index))
+  (declare (optimize speed)
+           (explicit-check n))
+  (cond ((not (typep n '(and fixnum unsigned-byte)))
+         (the unsigned-byte n)
          nil)
+        ((zerop n)
+         (copy-list list))
         (t
          (let ((head (dotted-nthcdr (1- n) list)))
-           (and (consp head)      ; there are at least n
-                (collect ((copy)) ; conses; copy!
+           (and (consp head)            ; there are at least n
+                (collect ((copy))       ; conses; copy!
                   (do ((trail list (cdr trail))
                        (head head (cdr head)))
                       ;; HEAD is n-1 conses ahead of TRAIL;
@@ -555,14 +552,17 @@
                     (copy (car trail)))))))))
 
 (defun nbutlast (list &optional (n 1))
-  (cond ((zerop n)
-         list)
-        ((not (typep n 'index))
+  (declare (optimize speed)
+           (explicit-check n))
+  (cond ((not (typep n '(and fixnum unsigned-byte)))
+         (the unsigned-byte n)
          nil)
+        ((zerop n)
+         list)
         (t
          (let ((head (dotted-nthcdr (1- n) list)))
-           (and (consp head)       ; there are more than n
-                (consp (cdr head)) ; conses.
+           (and (consp head)            ; there are more than n
+                (consp (cdr head))      ; conses.
                 ;; TRAIL trails by n cons to be able to
                 ;; cut the list at the cons just before.
                 (do ((trail list (cdr trail))
@@ -596,12 +596,6 @@
   "Change the CDR of CONS to X and return the CONS."
   (rplacd cons x))
 
-;;; The following are for use by SETF.
-
-(defun %rplaca (x val) (rplaca x val) val)
-
-(defun %rplacd (x val) (rplacd x val) val)
-
 ;;; Set the Nth element of LIST to NEWVAL.
 (defun %setnth (n list newval)
   (typecase n
@@ -1420,7 +1414,7 @@
                                  body-loop)))
                   `(defun ,(intern (format nil "%~A~{-~A~}~@[-~A~]" name funs variant))
                        (x list ,@funs)
-                     (declare (optimize speed (sb!c::verify-arg-count 0)))
+                     (declare (optimize speed (sb-c:verify-arg-count 0)))
                      ,@(when funs `((declare (function ,@funs)
                                              (dynamic-extent ,@funs))))
                      ,@(unless (member name '(member assoc adjoin rassoc))
