@@ -11,6 +11,26 @@
 
 (in-package "SB-PRETTY")
 
+;;; Ancestral types
+(defstruct (queued-op (:constructor nil) (:copier nil))
+  (posn 0 :type posn))
+
+(defstruct (block-end (:include queued-op) (:copier nil))
+  (suffix nil :type (or null simple-string)))
+
+(declaim (start-block))
+(defstruct (section-start (:include queued-op)
+                          (:constructor nil)
+                          (:copier nil))
+  (depth 0 :type index)
+  (section-end nil :type (or null newline block-end)))
+
+(defstruct (newline (:include section-start) (:copier nil))
+  (kind (missing-arg)
+        :type (member :linear :fill :miser :literal :mandatory)))
+(declaim (freeze-type newline)
+         (end-block))
+
 #-sb-fluid (declaim (inline index-posn posn-index posn-column))
 (defun index-posn (index stream)
   (declare (type index index) (type pretty-stream stream)
@@ -120,6 +140,7 @@
   (suffix-length 0 :type index)
   ;; The line number
   (section-start-line 0 :type index))
+(declaim (freeze-type logical-block))
 
 (defun really-start-logical-block (stream column prefix suffix)
   (let* ((blocks (pretty-stream-blocks stream))
@@ -213,7 +234,7 @@
          ,entry))))
 
 (defun enqueue-newline (stream kind)
-  (let* ((depth (length (pretty-stream-pending-blocks stream)))
+  (let* ((depth (pretty-stream-pending-blocks-length stream))
          (newline (enqueue stream newline :kind kind :depth depth)))
     (dolist (entry (pretty-stream-queue-tail stream))
       (when (and (not (eq newline entry))
@@ -227,6 +248,7 @@
                         (:copier nil))
   (kind (missing-arg) :type (member :block :current))
   (amount 0 :type fixnum))
+(declaim (freeze-type indentation))
 
 (defun enqueue-indent (stream kind amount)
   (enqueue stream indentation :kind kind :amount amount))
@@ -236,6 +258,7 @@
   (block-end nil :type (or null block-end))
   (prefix nil :type (or null simple-string))
   (suffix nil :type (or null simple-string)))
+(declaim (freeze-type block-start))
 
 (defun start-logical-block (stream prefix per-line-p suffix)
   ;; (In the PPRINT-LOGICAL-BLOCK form which calls us,
@@ -251,15 +274,15 @@
     (pretty-sout stream prefix 0 (length prefix)))
   (unless (typep suffix 'simple-string)
     (setq suffix (coerce suffix '(simple-array character (*)))))
-  (let* ((pending-blocks (pretty-stream-pending-blocks stream))
-         (start (enqueue stream block-start
-                         :prefix (and per-line-p prefix)
-                         :suffix suffix
-                         :depth (length pending-blocks))))
-    (setf (pretty-stream-pending-blocks stream)
-          (cons start pending-blocks))))
+  (let ((start (enqueue stream block-start
+                        :prefix (and per-line-p prefix)
+                        :suffix suffix
+                        :depth (pretty-stream-pending-blocks-length stream))))
+    (push start (pretty-stream-pending-blocks stream))
+    (incf (pretty-stream-pending-blocks-length stream))))
 
 (defun end-logical-block (stream)
+  (decf (pretty-stream-pending-blocks-length stream))
   (let* ((start (pop (pretty-stream-pending-blocks stream)))
          (suffix (block-start-suffix start))
          (end (enqueue stream block-end :suffix suffix)))
@@ -273,6 +296,7 @@
   (relativep nil :type (member t nil))
   (colnum 0 :type column)
   (colinc 0 :type column))
+(declaim (freeze-type queued-op)) ; and all subtypes
 
 (defun enqueue-tab (stream kind colnum colinc)
   (multiple-value-bind (sectionp relativep)
@@ -721,7 +745,7 @@ line break."
   ;; T iff one of the original entries.
   (initial-p (null *initial-pprint-dispatch-table*) :type boolean :read-only t)
   ;; and the associated function
-  (fun nil :type callable :read-only t))
+  (fun nil :type function-designator :read-only t))
 
 (declaim (freeze-type pprint-dispatch-entry))
 
@@ -780,7 +804,9 @@ line break."
 (defun copy-pprint-dispatch (&optional (table *print-pprint-dispatch*))
   (declare (type (or pprint-dispatch-table null) table))
   (let* ((orig (or table *initial-pprint-dispatch-table*))
-         (new (make-pprint-dispatch-table (copy-list (pp-dispatch-entries orig)))))
+         (new (make-pprint-dispatch-table (copy-list (pp-dispatch-entries orig))
+                                          (pp-dispatch-number-matchable-p orig)
+                                          (pp-dispatch-only-initial-entries orig))))
     (replace/eql-hash-table (pp-dispatch-cons-entries new)
                             (pp-dispatch-cons-entries orig))
     new))
@@ -788,20 +814,43 @@ line break."
 (defun pprint-dispatch (object &optional (table *print-pprint-dispatch*))
   (declare (type (or pprint-dispatch-table null) table))
   (let* ((table (or table *initial-pprint-dispatch-table*))
+         (possibly-matchable
+          (if (pp-dispatch-only-initial-entries table)
+              ;; Filters out SYMBOL, INSTANCE, FUNCTION, NUMBER, most arrays,
+              ;; and many other types very quickly. As horrible as the code emitted
+              ;; for this check is, it's still 3x to 4x faster than >= 6 funcalls.
+              ;; The inefficiency of the array test can be fixed by transforming
+              ;; it to something clever like
+              ;;  (and (> widetag #x80) (logbitp (ash widetag -2) #xNNNN)) ; some bit pattern
+              ;; at the source level, or in the backend.
+              ;; See !PPRINT-COLD-INIT for the possibly matchable types.
+              (typep object '(or cons
+                                 (and array (not (or string bit-vector)))
+                                 sb-impl::comma))
+              ;; We could do something similar to the POSSIBLY-MATCHABLE test
+              ;; based on tracking separately whether the user added any non-cons
+              ;; entries and cons entries. So if no non-cons entries were added,
+              ;; then the table can't match OBJECT if it is an atom and in the
+              ;; set of atoms matched by the preceding TYPEP test.
+              (or (not (numberp object))
+                  (pp-dispatch-number-matchable-p table))))
          (entry
-          (when (or (not (numberp object)) (pp-dispatch-number-matchable-p table))
+          (when possibly-matchable
             (let ((cons-entry
                    (and (consp object)
-                        (gethash (car object) (pp-dispatch-cons-entries table)))))
-              (if (not cons-entry)
-                  (dolist (entry (pp-dispatch-entries table) nil)
-                    (when (funcall (pprint-dispatch-entry-test-fn entry) object)
-                      (return entry)))
-                  (dolist (entry (pp-dispatch-entries table) cons-entry)
-                    (when (entry< entry cons-entry)
-                      (return cons-entry))
-                    (when (funcall (pprint-dispatch-entry-test-fn entry) object)
-                      (return entry))))))))
+                        (sb-impl::gethash/eql (car object) (pp-dispatch-cons-entries table) nil))))
+              (cond ((not cons-entry)
+                     (dolist (entry (pp-dispatch-entries table) nil)
+                       (when (funcall (pprint-dispatch-entry-test-fn entry) object)
+                         (return entry))))
+                    ((pp-dispatch-only-initial-entries table)
+                     cons-entry)
+                    (t
+                     (dolist (entry (pp-dispatch-entries table) cons-entry)
+                       (when (entry< entry cons-entry)
+                         (return cons-entry))
+                       (when (funcall (pprint-dispatch-entry-test-fn entry) object)
+                         (return entry)))))))))
     (if entry
         (values (pprint-dispatch-entry-fun entry) t)
         (values #'output-ugly-object nil))))
@@ -837,7 +886,7 @@ line break."
 ;; (cons (eql foo) cons) and (cons (eql foo) bit-vector) as two FOO entries.
 (defun set-pprint-dispatch (type function &optional
                             (priority 0) (table *print-pprint-dispatch*))
-  (declare (type (or null callable) function)
+  (declare (type function-designator function)
            (type real priority)
            (type pprint-dispatch-table table))
   (declare (explicit-check))
@@ -869,7 +918,8 @@ line break."
             (if function
                 (setf (gethash key hashtable) entry)
                 (remhash key hashtable))))
-        (setf (pp-dispatch-entries table)
+        (setf (pp-dispatch-only-initial-entries table) nil
+              (pp-dispatch-entries table)
               (let ((list (delete type (pp-dispatch-entries table)
                                   :key #'pprint-dispatch-entry-type
                                   :test #'equal)))
@@ -1442,11 +1492,16 @@ line break."
   ;; it's going to be set to a copy of *INITIAL-PP-D-T* below because
   ;; it's used in WITH-STANDARD-IO-SYNTAX, and condition reportery
   ;; possibly performed in the following extent may use W-S-IO-SYNTAX.
-  (setf *standard-pprint-dispatch-table* (make-pprint-dispatch-table))
+  (setf *standard-pprint-dispatch-table* (make-pprint-dispatch-table nil nil nil))
   (setf *initial-pprint-dispatch-table* nil)
-  (let ((*print-pprint-dispatch* (make-pprint-dispatch-table)))
+  (let ((*print-pprint-dispatch* (make-pprint-dispatch-table nil nil nil)))
     (/show0 "doing SET-PPRINT-DISPATCH for regular types")
-    (set-pprint-dispatch '(and array (not (or string bit-vector))) 'pprint-array)
+    ;; * PLEASE NOTE : If you change these definitions, then you may need to adjust
+    ;; the computation of POSSIBLY-MATCHABLE in PPRINT-DISPATCH.
+    ;;
+    ;; Assign a lower priority than for the cons entries below, making
+    ;; fewer type tests when dispatching.
+    (set-pprint-dispatch '(and array (not (or string bit-vector))) 'pprint-array -1)
     ;; MACRO-FUNCTION must have effectively higher priority than FBOUNDP.
     ;; The implementation happens to check identical priorities in the order added,
     ;; but that's unspecified behavior.  Both must be _strictly_ lower than the
@@ -1461,7 +1516,6 @@ line break."
     (set-pprint-dispatch 'sb-impl::comma 'pprint-unquoting-comma -3)
     ;; cons cells with interesting things for the car
     (/show0 "doing SET-PPRINT-DISPATCH for CONS with interesting CAR")
-
     (dolist (magic-form '((lambda pprint-lambda)
                           ((declare declaim) pprint-declare)
 
@@ -1513,7 +1567,8 @@ line break."
       ;; The sharing of dispatch entries is inconsequential.
       (set-pprint-dispatch `(cons (member ,@(ensure-list (first magic-form))))
                            (second magic-form)))
-    (setf *initial-pprint-dispatch-table* *print-pprint-dispatch*))
+    (setf (pp-dispatch-only-initial-entries *print-pprint-dispatch*) t
+          *initial-pprint-dispatch-table* *print-pprint-dispatch*))
 
   (setf *standard-pprint-dispatch-table*
         (copy-pprint-dispatch *initial-pprint-dispatch-table*))
