@@ -8,7 +8,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defvar *current-internal-error-context*)
 
@@ -16,11 +16,11 @@
       ((&optional (context '*current-internal-error-context*))
        &body body)
     (declare (ignorable context))
-    #!+(or x86 x86-64)
+    #+(or x86 x86-64)
     `(progn ,@body)
-    #!-(or x86 x86-64)
+    #-(or x86 x86-64)
     `(with-pinned-objects ((without-gcing
-                             (sb!di::code-object-from-context ,context)))
+                             (sb-di::code-object-from-context ,context)))
        ,@body))
 
 ;;;; OS-CONTEXT-T
@@ -60,7 +60,7 @@
 ;; or n32 ABI support.
 (defconstant kludge-big-endian-short-pointer-offset
   (+ 0
-     #!+(and mips big-endian (not 64-bit)) 1))
+     #+(and mips big-endian (not 64-bit)) 1))
 
 (declaim (inline context-pc-addr))
 (define-alien-routine ("os_context_pc_addr" context-pc-addr) (* unsigned)
@@ -130,3 +130,8 @@
                            (* kludge-big-endian-short-pointer-offset
                               n-word-bytes))
           new)))
+
+;;; Convert the descriptor into a SAP. The bits all stay the same, we just
+;;; change our notion of what we think they are.
+(declaim (inline descriptor-sap))
+(defun descriptor-sap (x) (int-sap (get-lisp-obj-address x)))
