@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!FORMAT")
+(in-package "SB-FORMAT")
 
 ;;;; FORMAT
 
@@ -19,7 +19,7 @@
 ;;; Also note that (DEFTYPE FORMAT-CONTROL) = (OR STRING FUNCTION).
 ;;; And it's possible that we could decide to install a closure as
 ;;; the fin-fun but I don't think that's necessary.
-(sb!kernel::!defstruct-with-alternate-metaclass fmt-control
+(sb-kernel:!defstruct-with-alternate-metaclass fmt-control
   :slot-names (string symbols memo)
   :constructor %make-fmt-control
   :superclass-name function
@@ -27,8 +27,6 @@
   :metaclass-constructor make-static-classoid
   :dd-type funcallable-structure)
 
-(declaim (freeze-type fmt-control))
-
 (defmethod print-object ((self fmt-control) stream)
   (print-unreadable-object (self stream :type t)
     (write-string (unparse-fmt-control self) stream)))
@@ -179,9 +177,9 @@
                             "~:@(~:C~)-FORMAT-DIRECTIVE-INTERPRETER"
                             char)))
         (directive '.directive) ; expose this var to the lambda. it's easiest
-        (directives (if lambda-list (car (last lambda-list)) (sb!xc:gensym "DIRECTIVES"))))
+        (directives (if lambda-list (car (last lambda-list)) (sb-xc:gensym "DIRECTIVES"))))
     `(setf
-       (aref *format-directive-interpreters* (sb!xc:char-code (char-upcase ,char)))
+       (aref *format-directive-interpreters* (sb-xc:char-code (char-upcase ,char)))
        (named-lambda ,defun-name (stream ,directive ,directives orig-args args)
          (declare (ignorable stream orig-args args))
          ,@(if lambda-list
@@ -194,7 +192,7 @@
                  ,@body))))))
 
 (defmacro def-format-interpreter (char lambda-list &body body)
-  (let ((directives (sb!xc:gensym "DIRECTIVES")))
+  (let ((directives (sb-xc:gensym "DIRECTIVES")))
     `(def-complex-format-interpreter ,char (,@lambda-list ,directives)
        ,@body
        ,directives)))
@@ -540,6 +538,11 @@
 (defun decimal-string (n)
   (write-to-string n :base 10 :radix nil :escape nil))
 
+;;; TODO: many of the the CHECK-MODIFIER calls in the directive interpreters
+;;; can be checked at tokenization time, which benefits from the fact that
+;;; a string is only tokenized once (unless evicted from cache).
+;;; Repeated calls using the same format control need not repeatedly check
+;;; for correctness of the string.
 (def-format-interpreter #\F (colonp atsignp params)
   (check-modifier "colon" colonp)
   (interpret-bind-defaults ((w nil) (d nil) (k nil) (ovf nil) (pad #\space))
@@ -569,13 +572,13 @@
      (prin1 number stream)
      nil)
     (t
-     (sb!impl::string-dispatch (single-float double-float)
+     (sb-impl::string-dispatch (single-float double-float)
          number
        (let ((spaceleft w))
-         (when (and w (or atsign (minusp (float-sign number))))
+         (when (and w (or atsign (= (float-sign-bit number) 1)))
            (decf spaceleft))
          (multiple-value-bind (str len lpoint tpoint)
-             (sb!impl::flonum-to-string (abs number) spaceleft d k)
+             (sb-impl::flonum-to-string (abs number) spaceleft d k)
            ;; if caller specifically requested no fraction digits, suppress the
            ;; optional trailing zero
            (when (and d (zerop d))
@@ -601,7 +604,7 @@
                   (when w
                     (dotimes (i spaceleft)
                       (write-char pad stream)))
-                  (if (minusp (float-sign number))
+                  (if (= (float-sign-bit number) 1)
                       (write-char #\- stream)
                       (when atsign
                         (write-char #\+ stream)))
@@ -637,9 +640,9 @@
   (if (case *read-default-float-format*
         ((short-float single-float)
          (typep number 'single-float))
-        ((double-float #!-long-float long-float)
+        ((double-float #-long-float long-float)
          (typep number 'double-float))
-        #!+long-float
+        #+long-float
         (long-float
          (typep number 'long-float)))
       #\e
@@ -666,22 +669,22 @@
   (if (or (float-infinity-p number)
           (float-nan-p number))
       (prin1 number stream)
-      (multiple-value-bind (num expt) (sb!impl::scale-exponent (abs number))
-        (let* ((k (if (= num 1.0) (1- k) k))
+      (multiple-value-bind (num expt) (sb-impl::scale-exponent (abs number))
+        (let* ((k (if (= num $1.0) (1- k) k))
                (expt (- expt k))
                (estr (decimal-string (abs expt)))
                (elen (if e (max (length estr) e) (length estr)))
                spaceleft)
           (when w
             (setf spaceleft (- w 2 elen))
-            (when (or atsign (minusp (float-sign number)))
+            (when (or atsign (= (float-sign-bit number) 1))
               (decf spaceleft)))
           (if (and w ovf e (> elen e))  ;exponent overflow
               (dotimes (i w) (write-char ovf stream))
               (let* ((fdig (if d (if (plusp k) (1+ (- d k)) d) nil))
                      (fmin (if (minusp k) 1 fdig)))
                 (multiple-value-bind (fstr flen lpoint tpoint)
-                    (sb!impl::flonum-to-string num spaceleft fdig k fmin)
+                    (sb-impl::flonum-to-string num spaceleft fdig k fmin)
                   (when (and d (zerop d)) (setq tpoint nil))
                   (when w
                     (decf spaceleft flen)
@@ -696,7 +699,7 @@
                          (dotimes (i w) (write-char ovf stream)))
                         (t (when w
                              (dotimes (i spaceleft) (write-char pad stream)))
-                           (if (minusp (float-sign number))
+                           (if (= (float-sign-bit number) 1)
                                (write-char #\- stream)
                                (if atsign (write-char #\+ stream)))
                            (when lpoint (write-char #\0 stream))
@@ -739,7 +742,7 @@
   (if (or (float-infinity-p number)
           (float-nan-p number))
       (prin1 number stream)
-      (multiple-value-bind (ignore n) (sb!impl::scale-exponent (abs number))
+      (multiple-value-bind (ignore n) (sb-impl::scale-exponent (abs number))
         (declare (ignore ignore))
         ;; KLUDGE: Default d if omitted. The procedure is taken directly from
         ;; the definition given in the manual, and is not very efficient, since
@@ -747,7 +750,7 @@
         ;; improve on this. -- rtoy?? 1998??
         (unless d
           (multiple-value-bind (str len)
-              (sb!impl::flonum-to-string (abs number))
+              (sb-impl::flonum-to-string (abs number))
             (declare (ignore str))
             (let ((q (if (= len 1) 1 (1- len))))
               (setq d (max q (min n 7))))))
@@ -776,12 +779,12 @@
     ;; thing, and at least the user shouldn't be surprised.
     (setq number (coerce number 'single-float)))
   (if (floatp number)
-      (let* ((signstr (if (minusp (float-sign number))
+      (let* ((signstr (if (= (float-sign-bit number) 1)
                           "-"
                           (if atsign "+" "")))
              (signlen (length signstr)))
         (multiple-value-bind (str strlen ig2 ig3 pointplace)
-            (sb!impl::flonum-to-string (abs number) nil d nil)
+            (sb-impl::flonum-to-string (abs number) nil d nil)
           (declare (ignore ig2 ig3 strlen))
           (when colon
             (write-string signstr stream))
@@ -859,18 +862,18 @@
     (write-string spaces stream :end n)))
 
 (defun format-relative-tab (stream colrel colinc)
-  (if (sb!pretty:pretty-stream-p stream)
+  (if (sb-pretty:pretty-stream-p stream)
       (pprint-tab :line-relative colrel colinc stream)
-      (let* ((cur (sb!impl::charpos stream))
+      (let* ((cur (sb-impl::charpos stream))
              (spaces (if (and cur (plusp colinc))
                          (- (* (ceiling (+ cur colrel) colinc) colinc) cur)
                          colrel)))
         (output-spaces stream spaces))))
 
 (defun format-absolute-tab (stream colnum colinc)
-  (if (sb!pretty:pretty-stream-p stream)
+  (if (sb-pretty:pretty-stream-p stream)
       (pprint-tab :line colnum colinc stream)
-      (let ((cur (sb!impl::charpos stream)))
+      (let ((cur (sb-impl::charpos stream)))
         (cond ((null cur)
                (write-string "  " stream))
               ((< cur colnum)
@@ -1136,7 +1139,7 @@
               (when (and first-semi (directive-colonp first-semi))
                 (interpret-bind-defaults
                     ((extra 0)
-                     (len (or (sb!impl::line-length stream) 72)))
+                     (len (or (sb-impl::line-length stream) 72)))
                     (directive-params first-semi)
                   (setf newline-string
                         (with-simple-output-to-string (stream)
@@ -1173,7 +1176,7 @@
                      mincol))
          (padding (+ (- length chars) (* num-gaps minpad))))
     (when (and newline-prefix
-               (> (+ (or (sb!impl::charpos stream) 0)
+               (> (+ (or (sb-impl::charpos stream) 0)
                      length extra-space)
                   line-len))
       (write-string newline-prefix stream))
