@@ -61,9 +61,11 @@
   (sb-ext:without-package-locks
    (dolist (pkg (list-all-packages))
      (let ((name (package-name pkg)))
-       (unless (member name '("KEYWORD" "COMMON-LISP"  "COMMON-LISP-USER"
-                              "SB-COLD" "SB!XC")
-                       :test #'string=)
+       (unless (or (member name '("KEYWORD" "COMMON-LISP"  "COMMON-LISP-USER"
+                                  "XC-STRICT-CL" "SB-COLD" "SB-XC")
+                           :test #'string=)
+                   #+swank
+                   (search "SWANK" name))
          ;; This also removes nicknames SEQUENCE and SB-C-CALL.
          (rename-package pkg (concatenate 'string "HOST-" name)))))))
 
@@ -120,9 +122,9 @@
               (when (string= (car entry) 'print-type)
                 (let ((symbol (intern name (cddr entry))))
                   ;; The parallelized build performs this twice: once from
-                  ;;interpreted load, again from compilation.
+                  ;; interpreted load, again from compilation.
                   ;; So don't wrap more than once.
-                  (unless (sb-kernel:closurep (fdefinition symbol))
+                  (unless (sb-impl::closurep (fdefinition symbol))
                     (setf (fdefinition symbol) combined))))))
           (dolist (entry *host-format-functions*)
             (when (string= (car entry) name)
@@ -146,9 +148,11 @@
   (name (error "missing PACKAGE-DATA-NAME datum"))
   ;; a doc string
   (doc (error "missing PACKAGE-DOC datum"))
+  ;; a list of string designators for shadowing symbols
+  shadow
   ;; a tree containing names for exported symbols which'll be set up at package
   ;; creation time, and NILs, which are ignored. (This is a tree in order to
-  ;; allow constructs like '("ENOSPC" #!+LINUX ("EDQUOT" "EISNAM" "ENAVAIL"
+  ;; allow constructs like '("ENOSPC" #+LINUX ("EDQUOT" "EISNAM" "ENAVAIL"
   ;; "EREMOTEIO")) to be used in initialization. NIL entries in the tree are
   ;; ignored for the same reason of notational convenience.)
   export
@@ -165,6 +169,79 @@
   ;; which this package uses
   use)
 
+;;; A symbol in the "shadows" list ALWAYS refers to the symbol
+;;; in SB-XC when unqualified. Each symbol uncrosses to itself.
+;;; I'm taking the stance that since we don't seem to have any calls to
+;;; PLUSP or MINUSP involving target floatnums, we don't need to provide
+;;; alternate symbols.
+(defparameter *shadows*
+  '("FLOAT" "SHORT-FLOAT" "SINGLE-FLOAT" "DOUBLE-FLOAT" "LONG-FLOAT"
+    "REAL" "COMPLEX" "NUMBER"
+    ;; "RATIONAL" is here for the same reason are the preceding:
+    ;; we don't want to mess up all tests of the form (IF (EQ X 'RATIONAL) ...)
+    ;; or worry about the package of the symbol we're testing (since identity matters).
+    ;; But we also need to logically shadow #'RATIONAL which would not be legal
+    ;; if it refers to the standard CL:RATIONAL symbol,
+    ;; but we need not to harm the standard type specifier '(RATIONAL).
+    "RATIONAL"
+    ;; Since we're shadowing the types, it also makes sense to shadow
+    ;; the predicates lest it be confusing to have to write SB-XC:
+    ;; in front of them but not in front of the type.
+    ;; RATIONALP isn't here because its behavior is unchanged.
+    "FLOATP" "REALP" "COMPLEXP" "NUMBERP"
+    "COERCE" "EXP" "EXPT" "LOG" "SIGNUM" "IMAGPART" "REALPART"
+    "ZEROP" "ABS" "SIGNUM" "FLOAT-SIGN"
+    "CEILING" "FLOOR" "ROUND" "TRUNCATE" "MOD" "REM"
+    ;; We always want irrational functions to use target floats.
+    "ACOS" "ACOSH" "ASIN" "ASINH" "ATAN" "ATANH"  "CIS" "CONJUGATE"
+    "COS" "COSH"  "FCEILING" "FFLOOR" "FROUND" "FTRUNCATE"
+    "PHASE" "RATIONALIZE" "SIN" "SINH" "SQRT" "TAN" "TANH"
+    ;;
+    "SXHASH" ; must package-qualify if you mean CL:SXHASH
+    ;;
+    "BYTE" "BYTE-POSITION" "BYTE-SIZE"
+    "DPB" "LDB" "LDB-TEST"
+    "DEPOSIT-FIELD" "MASK-FIELD"))
+
+;;; A symbol in the "dual personality" list refers to the symbol in CL unless
+;;; package-prefixed with SB-XC:.  The main reason for not putting these
+;;; in the *shadows* list is that it's not worth trying to handle, or inefficient
+;;; to handle the general case of +,-,*,/ and the comparators.
+;;; i.e. the code compiled in make-host-1 by the host compiler would be
+;;; that much less efficient by always having to use the intercepted function)
+;;; We're also not handling 1+ or 1- or INCF, DECF.
+;;; It's unlikely that a host floating-pointer value could sneak through
+;;; to one of the un-intercepted functions given the prohibition against
+;;; using floating-point literals and that almost all other functions
+;;; are intercepted. Granted there are some roundabout ways to spell a
+;;; floating-point number that can not be detected, such as:
+;;;   (* 50 (hash-table-rehash-threshold (make-hash-table)))
+;;; because we are not intercepting '*.
+(defparameter *dual-personality-math-symbols*
+  '("+" "-" "*" "/" "=" "/=" "<" "<=" ">" ">=" "MIN" "MAX"
+    ;; We've gotten along quite well without an alter-ego for FIXNUM,
+    ;; but now some s-expressions mentioning the type FIXNUM are fed
+    ;; to the host for evaluation, and a type-checking host (such as we are)
+    ;; croaks if an argument exceeds the host's notion of fixnum.
+    ;; Get around it by changing those uses of fixnum to SB-XC:FIXNUM.
+    "FIXNUM"
+    ))
+
+;;; When playing such tricky package games, it's best to have the symbols that
+;;; are visible by default, i.e. XC-STRICT-CL:NAME, have no definition,
+;;; and the expressly qualified (with SB-XC:) symbols have definitions.
+;;; This idiom makes you pick one or the other of CL:THING or SB-XC:THING,
+;;; and not ever just get one at random.
+;;; In fact I especially don't like the magic byte specifier hacks. It would be
+;;; safer and clearer to have no definition associated with the symbols that you
+;;; see by default, so that using them by accident fails.
+(defparameter *undefineds*
+  '("SYMBOL-PACKAGE"
+    ;; Float decoding: don't want to see these used either.
+    "DECODE-FLOAT" "INTEGER-DECODE-FLOAT"
+    "FLOAT-DIGITS" "FLOAT-PRECISION" "FLOAT-RADIX"
+    "SCALE-FLOAT"))
+
 ;; The running-in-the-host-Lisp Python cross-compiler defines its
 ;; own versions of a number of functions which should not overwrite
 ;; host-Lisp functions. Instead we put them in a special package.
@@ -173,10 +250,9 @@
 ;; forth in this package is that they run in the host and affect the
 ;; compilation of the target.
 ;;
-;; FIXME: this package should have only one name, not two,
-;; and its one name should be SBCL, but changing it to that
-;; would entail touching about 900 lines.
-(let ((package-name "SB!XC"))
+(let ((package-name "SB-XC"))
+  (dolist (name (append (append *undefineds* *dual-personality-math-symbols*)))
+    (export (intern name package-name) package-name))
   (dolist (name '(;; the constants (except for T and NIL which have
                   ;; a specially hacked correspondence between
                   ;; cross-compilation host Lisp and target Lisp)
@@ -240,10 +316,9 @@
                   "SHORT-FLOAT-NEGATIVE-EPSILON"
                   "SINGLE-FLOAT-EPSILON"
                   "SINGLE-FLOAT-NEGATIVE-EPSILON"
+                  "*READ-DEFAULT-FLOAT-FORMAT*"
 
-                  ;; everything else which needs a separate
-                  ;; existence in xc and target
-                  "BYTE" "BYTE-POSITION" "BYTE-SIZE"
+                  "ARRAY-ELEMENT-TYPE"
                   "CHAR-CODE"
                   "CODE-CHAR"
                   "COMPILE-FILE"
@@ -258,17 +333,16 @@
                   "DEFINE-MODIFY-MACRO"
                   "DEFINE-SETF-EXPANDER"
                   "DEFMACRO" "DEFSETF" "DEFSTRUCT" "DEFTYPE"
-                  "DEPOSIT-FIELD" "DPB"
                   "GENSYM" "*GENSYM-COUNTER*"
                   "GET-SETF-EXPANSION"
-                  "LDB" "LDB-TEST"
                   "LISP-IMPLEMENTATION-TYPE" "LISP-IMPLEMENTATION-VERSION"
                   "MACRO-FUNCTION"
                   "MACROEXPAND" "MACROEXPAND-1" "*MACROEXPAND-HOOK*"
+                  "MAKE-ARRAY"
                   "MAKE-LOAD-FORM"
                   "MAKE-LOAD-FORM-SAVING-SLOTS"
-                  "MASK-FIELD"
                   "PROCLAIM"
+                  "SIMPLE-VECTOR"
                   "SPECIAL-OPERATOR-P"
                   "SUBTYPEP"
                   "TYPE-OF" "TYPEP"
@@ -277,17 +351,57 @@
                   "WITH-COMPILATION-UNIT"))
       (export (intern name package-name) package-name)))
 
-;; Symbols that we want never to accidentally see the host's definition of.
-(defparameter *shadowing-imports*
-  (mapcar (lambda (name) (find-symbol name "SB!XC"))
-          '("BYTE" "BYTE-POSITION" "BYTE-SIZE"
-            "DPB" "LDB" "LDB-TEST"
-            "DEPOSIT-FIELD" "MASK-FIELD")))
+(defun count-symbols (pkg)
+  (let ((n 0))
+    (do-external-symbols (s pkg n)
+      (declare (ignorable s))
+      (incf n))))
 
-(let ((package-data-list (read-from-file "package-data-list.lisp-expr")))
+;;; Build a new package that exports a not-necessarily-strict subset of
+;;; what the host CL exports. This deals with hosts that have too many
+;;; symbols exported froM CL.
+(let ((cl-model-package (make-package "XC-STRICT-CL" :use nil)))
+  (flet ((new-external (x package &aux (s (intern x package)))
+           (export s package)
+           s)
+         (reexport (x)
+           (import x cl-model-package)
+           (export x cl-model-package)))
+    (reexport (list nil))
+    (dolist (string (read-from-file "common-lisp-exports.lisp-expr"))
+      (unless (string= string "NIL") ; already done
+        (cond ((member string *undefineds* :test #'string=)
+               (new-external string cl-model-package))
+              ((find string *shadows* :test #'string=)
+               (reexport (new-external string "SB-XC")))
+              ((find-symbol string "CL")
+               (reexport (find-symbol string "CL")))
+              (t
+               (warn "No symbol named ~S in host CL package!" string)
+               (new-external string cl-model-package)))))))
+
+;;; Snapshot so that we can ascertain in genesis that nothing new got interned
+;;; in the standardized packages.
+(defun compute-cl-package-symbol-counts ()
+  (mapcar (lambda (x) (cons x (count-symbols x)))
+          '("XC-STRICT-CL" "SB-XC")))
+
+(defvar *package-symbol-counts* (compute-cl-package-symbol-counts))
+(defun check-no-new-cl-symbols ()
+  (assert (equal *package-symbol-counts* (compute-cl-package-symbol-counts))))
+
+(defun create-target-packages (package-data-list)
   (labels ((flatten (tree)
-             (mapcan (lambda (x) (if (listp x) (flatten x) (list x)))
-                     tree)))
+             (let ((result (mapcan (lambda (x) (if (listp x) (flatten x) (list x)))
+                                   tree)))
+               (when (< (length (remove-duplicates result :test 'equal))
+                        (length result))
+                 (error "Duplicates in package-data-list: ~a~%"
+                        (mapcon (lambda (x)
+                                  (when (member (car x) (cdr x) :test 'equal)
+                                    (list (car x))))
+                                result)))
+               result)))
 
     (hide-host-packages)
 
@@ -295,11 +409,10 @@
     ;; can without referring to any other packages.
     (dolist (package-data package-data-list)
       (let* ((name (package-data-name package-data))
-             (package (make-package
-                       name
-                       :nicknames (list (concatenate 'string "SB!" (subseq name 3)))
-                       :use nil)))
-        (shadowing-import *shadowing-imports* package)
+             (package (make-package name :use nil)))
+        ;; Walk the tree of shadowing names
+        (dolist (string (flatten (package-data-shadow package-data)))
+          (shadow string package))
         ;; Walk the tree of exported names, exporting each name.
         (dolist (string (flatten (package-data-export package-data)))
           (export (intern string package) package))))
@@ -307,7 +420,9 @@
     ;; Now that all packages exist, we can set up package-package
     ;; references.
     (dolist (package-data package-data-list)
-      (use-package (package-data-use package-data)
+      (use-package (substitute "XC-STRICT-CL" "CL"
+                               (package-data-use package-data)
+                               :test 'string=)
                    (package-data-name package-data))
       (dolist (sublist (package-data-import-from package-data))
         (let ((from-package (first sublist)))
@@ -354,21 +469,30 @@
           (reexport x))
         (assert (= (length done) (length package-data-list)))))))
 
+(export '*undefined-fun-whitelist*)
+(defvar *undefined-fun-whitelist* (make-hash-table :test 'equal))
+(let ((list
+       (with-open-file (data (prepend-genfile-path "package-data-list.lisp-expr"))
+         ;; There's no need to use the precautionary READ-FROM-FILE function
+         ;; with package-data-list because it is not a customization file.
+         (create-target-packages (let ((*readtable* *xc-readtable*)) (read data)))
+         (let ((*readtable* *xc-readtable*)) (read data)))))
+  (dolist (name (apply #'append list))
+    (setf (gethash name *undefined-fun-whitelist*) t)))
+
+(defvar *asm-package-use-list*
+  '("SB-ASSEM" "SB-DISASSEM"
+    "SB-INT" "SB-EXT" "SB-KERNEL" "SB-VM"
+    "SB-SYS" ; for SAP accessors
+    ;; Dependence of the assembler on the compiler feels a bit backwards,
+    ;; but assembly needs TN-SC, TN-OFFSET, etc. because the compiler
+    ;; doesn't speak the assembler's language. Rather vice-versa.
+    "SB-C"))
 (defun make-assembler-package (pkg-name)
   (when (find-package pkg-name)
     (delete-package pkg-name))
   (let ((pkg (make-package pkg-name
-                           :nicknames
-                           (list (concatenate 'string "SB!" (subseq pkg-name 3)))
-                           :use '("CL" "SB-INT" "SB-EXT" "SB-KERNEL" "SB-VM"
-                                  "SB-SYS" ; for SAP accessors
-                                  ;; Dependence of the assembler on the compiler
-                                  ;; feels a bit backwards, but assembly needs
-                                  ;; TN-SC, TN-OFFSET, etc. because the compiler
-                                  ;; doesn't speak the assembler's language.
-                                  ;; Rather vice-versa.
-                                  "SB-C"))))
-    (shadowing-import *shadowing-imports* pkg)
+                           :use (cons "XC-STRICT-CL" (cddr *asm-package-use-list*)))))
     ;; Both SB-ASSEM and SB-DISASSEM export these two symbols.
     ;; Neither is shadowing-imported. If you need one, package-qualify it.
     (shadow '("SEGMENT" "MAKE-SEGMENT") pkg)
@@ -380,10 +504,40 @@
 (make-assembler-package (backend-asm-package-name))
 
 (defun package-list-for-genesis ()
-  (append (read-from-file "package-data-list.lisp-expr")
+  (append (let ((*readtable* *xc-readtable*))
+            (read-from-file "package-data-list.lisp-expr" nil))
           (let ((asm-package (backend-asm-package-name)))
-            (list (make-package-data
-                   :name asm-package
-                   :use (mapcar 'package-name
-                                (package-use-list asm-package))
-                   :doc nil)))))
+            (list (make-package-data :name asm-package
+                                     :use (list* "CL" *asm-package-use-list*)
+                                     :doc nil)))))
+
+;;; Not all things shown by this are actually unused. Some get removed
+;;; by the tree-shaker as intended.
+#+nil
+(defun show-unused-exports (&aux nonexistent uninteresting)
+  (dolist (entry (with-open-file (f "package-data-list.lisp-expr") (read f)))
+    (let ((pkg (find-package (package-data-name entry))))
+      (dolist (string (mapcan (lambda (x) (if (stringp x) (list x) x))
+                              (package-data-export entry)))
+        (unless (or (string= string "!" :end1 1) (string= string "*!" :end1 2))
+          (let ((s (find-symbol string pkg)))
+            (cond ((not s)
+                   (push (cons pkg string) nonexistent))
+                  ((and (not (boundp s))
+                        (not (sb-kernel:symbol-info s))
+                        (not (gethash s sb-c::*backend-parsed-vops*)))
+                   (push s uninteresting))))))))
+  (format t "~&Nonexistent:~%")
+  (dolist (x nonexistent)
+    (format t "  ~a ~a~%" (package-name (car x)) (cdr x)))
+  (format t "~&Possibly uninteresting:~%")
+  ;; FIXME: prints some things that it shouldn't as "uninteresting"
+  ;; including but not limited to:
+  ;;   - alien struct slot names
+  ;;   - catch tag names (e.g. 'TOPLEVEL-CATCHER)
+  ;;   - declarations
+  ;;   - restart names
+  ;;   - object-not-<type>-error
+  ;;   - markers such as SB-SYS:MACRO (in lexenvs)
+  (dolist (x uninteresting)
+    (format t "  ~s~%" x)))
