@@ -65,15 +65,19 @@
       ;; Ensure that we're reading the correct variant of the file
       ;; in case there is more than one set of floating-point formats.
       (assert (eq (read stream) :default))
-      (let ((*package* (find-package "SB-KERNEL")))
-        (dolist (expr (read stream))
-          (destructuring-bind (fun args result) expr
-            (let ((actual (apply fun (sb-int:ensure-list args))))
-              (unless (eql actual result)
-                (#+sb-devel cerror #+sb-devel ""
-                 #-sb-devel format #-sb-devel t
-                 "FLOAT CACHE LINE ~S vs COMPUTED ~S~%"
-                 expr actual)))))))))
+      (sb-kernel::with-float-traps-masked (:overflow :divide-by-zero)
+        (let ((*package* (find-package "SB-KERNEL")))
+          (dolist (expr (read stream))
+            (destructuring-bind (fun args . result) expr
+              (let ((result (if (eq (first result) 'sb-kernel::&values)
+                                (rest result)
+                                result))
+                    (actual (multiple-value-list (apply fun (sb-int:ensure-list args)))))
+                (unless (equalp actual result)
+                  (#+sb-devel cerror #+sb-devel ""
+                   #-sb-devel format #-sb-devel t
+                   "FLOAT CACHE LINE ~S vs COMPUTED ~S~%"
+                   expr actual))))))))))
 
 (when (if (boundp '*compile-files-p*) *compile-files-p* t)
   (with-open-file (output "output/cold-vop-usage.txt" :if-does-not-exist nil)
@@ -164,7 +168,7 @@
                          ;; Maybe we should escalate more warnings to errors
                          ;; (see HANDLER-BIND for SIMPLE-WARNING below)
                          ;; rather than asking what to do here?
-                         #+x86-64 ;; this should complete without warnings
+                         #+(or x86 x86-64) ;; these should complete without warnings
                          (cerror "Ignore warnings" "Compile completed with warnings")))
                   (unless (handler-bind
                               ((sb-kernel:redefinition-with-defgeneric
@@ -176,12 +180,13 @@
 
   (let ((*compile-print* nil))
     (dolist (group sources)
-      (handler-bind ((simple-warning
+      (handler-bind ((#+x86-64 warning #-x86-64 simple-warning
                       (lambda (c)
                         ;; escalate "undefined variable" warnings to errors.
                         ;; There's no reason to allow them in our code.
-                        (when (search "undefined variable"
-                                      (write-to-string c :escape nil))
+                        (when (and #-x86-64 ; Don't allow any warnings on x86-64.
+                                   (search "undefined variable"
+                                           (write-to-string c :escape nil)))
                           (cerror "Finish warm compile ignoring the problem" c)))))
         (with-compilation-unit () (do-srcs group)))))))
 
