@@ -177,6 +177,7 @@
   (length 0 :type disassem-length)               ; in bytes
 
   (default-printer nil :type list))
+(declaim (freeze-type arg instruction-format))
 
 ;;; A FUNSTATE holds the state of any arguments used in a disassembly
 ;;; function. It is a 2-level alist. The outer list maps each ARG to
@@ -558,349 +559,6 @@
            `((lookup-label ,(maybe-listify numeric-forms)))
            numeric-forms)))))
 
-(defun find-printer-fun (printer-source args cache *current-instruction-flavor*)
-  (let* ((source (preprocess-printer printer-source args))
-         (funstate (make-funstate args))
-         (forms (let ((*!temp-var-counter* 0))
-                  (compile-printer-list source funstate)))
-         (bindings (make-arg-temp-bindings funstate))
-         (guts `(let* ,bindings ,@forms))
-         (sub-table (assq :printer cache)))
-    (or (cdr (assoc guts (cdr sub-table) :test #'equal))
-        (let ((template
-     `(named-lambda (inst-printer ,@*current-instruction-flavor*)
-        (chunk inst stream dstate
-               &aux (chunk (truly-the dchunk chunk))
-                    (inst (truly-the instruction inst))
-                    (stream (truly-the (or null stream) stream))
-                    (dstate (truly-the disassem-state dstate)))
-       (macrolet ((local-format-arg (arg fmt)
-                    `(funcall (formatter ,fmt) stream ,arg)))
-         ;; All the LOCAL-foo functions print nothing if stream is nil
-         (flet ((local-tab-to-arg-column ()
-                  (tab (dstate-argument-column dstate) stream))
-                (local-print-name ()
-                  (when stream (princ (inst-print-name inst) stream)))
-                (local-print-arg-separator ()
-                  (when stream (princ ", " stream)))
-                (local-write-char (ch)
-                  (when stream (write-char ch stream)))
-                (local-princ-symbol (thing)
-                  ;; This special case is reqired so that decoding x86 JMP and CMOV
-                  ;; to an instruction model does not insert a superfluous symbol.
-                  ;; I hope this is right. We really only want to elide symbols
-                  ;; that are not inside a :COND or other selector.
-                  (when stream (princ thing stream)))
-                (local-princ (thing)
-                  (if stream
-                      (princ thing stream)
-                      (push thing (dstate-operands dstate))))
-                (local-princ16 (thing)
-                  (if stream
-                      (princ16 thing stream)
-                      (push thing (dstate-operands dstate))))
-                (local-call-arg-printer (arg printer)
-                  (funcall printer arg stream dstate))
-                (local-call-global-printer (fun)
-                  (funcall fun chunk inst stream dstate))
-                (local-filtered-value (offset)
-                  (declare (type filtered-value-index offset))
-                  (aref (dstate-filtered-values dstate) offset))
-                (local-extract (bytespec)
-                  (dchunk-extract chunk bytespec))
-                (lookup-label (lab)
-                  (or (gethash lab (dstate-label-hash dstate))
-                      lab))
-                (adjust-label (val adjust-fun)
-                  (funcall adjust-fun val dstate)))
-           (declare (ignorable #'local-tab-to-arg-column
-                               #'local-print-name #'local-print-arg-separator
-                               #'local-princ #'local-princ16 #'local-princ-symbol
-                               #'local-write-char
-                               #'local-call-arg-printer
-                               #'local-call-global-printer
-                               #'local-extract
-                               #'local-filtered-value
-                               #'lookup-label #'adjust-label)
-                    (inline local-tab-to-arg-column
-                            local-print-arg-separator
-                            local-princ local-princ16 local-princ-symbol
-                            local-call-arg-printer local-call-global-printer
-                            local-filtered-value local-extract
-                            lookup-label adjust-label))
-           :body)))))
-          (cdar (push (cons guts (compile nil (subst guts :body template)))
-                      (cdr sub-table)))))))
-
-(defun preprocess-test (subj form args)
-  (multiple-value-bind (subj test)
-      (if (and (consp form) (symbolp (car form)) (not (keywordp (car form))))
-          (values (car form) (cdr form))
-          (values subj form))
-    (let ((key (if (consp test) (car test) test))
-          (body (if (consp test) (cdr test) nil)))
-      (case key
-        (:constant
-         (if (null body)
-             ;; If no supplied constant values, just any constant is ok,
-             ;; just see whether there's some constant value in the arg.
-             (not
-              (null
-               (arg-value
-                (or (find subj args :key #'arg-name)
-                    (pd-error "unknown argument ~S" subj)))))
-             ;; Otherwise, defer to run-time.
-             form))
-        ((:or :and :not)
-         (recons
-          form
-          subj
-          (recons
-           test
-           key
-           (sharing-mapcar
-            (lambda (sub-test)
-              (preprocess-test subj sub-test args))
-            body))))
-        (t form)))))
-
-(defun preprocess-conditionals (printer args)
-  (if (atom printer)
-      printer
-      (case (car printer)
-        (:unless
-         (preprocess-conditionals
-          `(:cond ((:not ,(nth 1 printer)) ,@(nthcdr 2 printer)))
-          args))
-        (:when
-         (preprocess-conditionals `(:cond (,(cdr printer))) args))
-        (:if
-         (preprocess-conditionals
-          `(:cond (,(nth 1 printer) ,(nth 2 printer))
-                  (t ,(nth 3 printer)))
-          args))
-        (:cond
-         (recons
-          printer
-          :cond
-          (sharing-mapcar
-           (lambda (clause)
-             (let ((filtered-body
-                    (sharing-mapcar
-                     (lambda (sub-printer)
-                       (preprocess-conditionals sub-printer args))
-                     (cdr clause))))
-               (recons
-                clause
-                (preprocess-test (find-first-field-name filtered-body)
-                                 (car clause)
-                                 args)
-                filtered-body)))
-           (cdr printer))))
-        (quote printer)
-        (t
-         (sharing-mapcar
-          (lambda (sub-printer)
-            (preprocess-conditionals sub-printer args))
-          printer)))))
-
-;;; Return a version of the disassembly-template PRINTER with
-;;; compile-time tests (e.g. :constant without a value), and any
-;;; :CHOOSE operators resolved properly for the args ARGS.
-;;;
-;;; (:CHOOSE Sub*) simply returns the first Sub in which every field
-;;; reference refers to a valid arg.
-(defun preprocess-printer (printer args)
-  (preprocess-conditionals (preprocess-chooses printer args) args))
-
-;;; Return the first non-keyword symbol in a depth-first search of TREE.
-(defun find-first-field-name (tree)
-  (cond ((null tree)
-         nil)
-        ((and (symbolp tree) (not (keywordp tree)))
-         tree)
-        ((atom tree)
-         nil)
-        ((eq (car tree) 'quote)
-         nil)
-        (t
-         (or (find-first-field-name (car tree))
-             (find-first-field-name (cdr tree))))))
-
-(defun preprocess-chooses (printer args)
-  (cond ((atom printer)
-         printer)
-        ((eq (car printer) :choose)
-         (pick-printer-choice (cdr printer) args))
-        (t
-         (sharing-mapcar (lambda (sub) (preprocess-chooses sub args))
-                         printer))))
-
-;;;; some simple functions that help avoid consing when we're just
-;;;; recursively filtering things that usually don't change
-
-(defun sharing-mapcar (fun list)
-  (declare (type function fun))
-  "A simple (one list arg) mapcar that avoids consing up a new list
-  as long as the results of calling FUN on the elements of LIST are
-  eq to the original."
-  (and list
-       (recons list
-                     (funcall fun (car list))
-                     (sharing-mapcar fun (cdr list)))))
-
-(defun all-arg-refs-relevant-p (printer args)
-  (cond ((or (null printer) (keywordp printer) (eq printer t))
-         t)
-        ((symbolp printer)
-         (find printer args :key #'arg-name))
-        ((listp printer)
-         (every (lambda (x) (all-arg-refs-relevant-p x args))
-                printer))
-        (t t)))
-
-(defun pick-printer-choice (choices args)
-  (dolist (choice choices
-           (pd-error "no suitable choice found in ~S" choices))
-    (when (all-arg-refs-relevant-p choice args)
-      (return choice))))
-
-(defun compile-printer-list (sources funstate)
-  (when sources
-    (cons (compile-printer-body (car sources) funstate)
-          (compile-printer-list (cdr sources) funstate))))
-
-(defun compile-printer-body (source funstate)
-  (cond ((null source)
-         nil)
-        ((eq source :name)
-         `(local-print-name))
-        ((eq source :tab)
-         `(local-tab-to-arg-column))
-        ((keywordp source)
-         (pd-error "unknown printer element: ~S" source))
-        ((symbolp source)
-         (compile-print source funstate))
-        ((atom source)
-         ;; Skip the argument separator if disassembling into a model.
-         ;; The ", " string is quite consistent amongst backends.
-         ;; Making this any more general - such as never printing strings
-         ;; into a model - would be prone to information loss.
-         (if (and (stringp source) (string= source ", "))
-             `(local-print-arg-separator)
-             `(local-princ ',source)))
-        ((eq (car source) 'quote)
-         `(,(if (symbolp (cadr source)) 'local-princ-symbol 'local-princ) ,source))
-        ((eq (car source) :using)
-         (let ((f (cadr source)))
-          (unless (typep f '(or string (cons (eql function) (cons symbol null))))
-            (pd-error "The first arg to :USING must be a string or #'function."))
-          (compile-print (caddr source) funstate f)))
-        ((eq (car source) :plus-integer)
-         ;; prints the given field proceed with a + or a -
-         (let ((form
-                (arg-value-form (arg-or-lose (cadr source) funstate)
-                                funstate
-                                :numeric)))
-           `(progn
-              (when (>= ,form 0)
-                (local-write-char #\+))
-              (local-princ ,form))))
-        ((eq (car source) 'function)
-         `(local-call-global-printer ,source))
-        ((eq (car source) :cond)
-         `(cond ,@(mapcar (lambda (clause)
-                            `(,(compile-test (find-first-field-name
-                                              (cdr clause))
-                                             (car clause)
-                                             funstate)
-                              ,@(compile-printer-list (cdr clause)
-                                                      funstate)))
-                          (cdr source))))
-        ;; :IF, :UNLESS, and :WHEN are replaced by :COND during preprocessing
-        (t
-         `(progn ,@(compile-printer-list source funstate)))))
-
-(defun compile-print (arg-name funstate &optional printer)
-  (let* ((arg (arg-or-lose arg-name funstate))
-         (printer (or printer (arg-printer arg))))
-    (etypecase printer
-      (string
-       `(local-format-arg ,(arg-value-form arg funstate) ,printer))
-      (vector
-       `(local-princ (aref ,printer ,(arg-value-form arg funstate :numeric))))
-      ((or function (cons (eql function)))
-       `(local-call-arg-printer ,(arg-value-form arg funstate) ,printer))
-      (boolean
-       `(,(if (arg-use-label arg) 'local-princ16 'local-princ)
-         ,(arg-value-form arg funstate))))))
-
-(defun compare-fields-form (val-form-1 val-form-2)
-  (flet ((listify-fields (fields)
-           (cond ((symbolp fields) fields)
-                 ((every #'constantp fields) `',fields)
-                 (t `(list ,@fields)))))
-    (cond ((or (symbolp val-form-1) (symbolp val-form-2))
-           `(equal ,(listify-fields val-form-1)
-                   ,(listify-fields val-form-2)))
-          (t
-           `(and ,@(mapcar (lambda (v1 v2) `(= ,v1 ,v2))
-                           val-form-1 val-form-2))))))
-
-(defun compile-test (subj test funstate)
-  (when (and (consp test) (symbolp (car test)) (not (keywordp (car test))))
-    (setf subj (car test)
-          test (cdr test)))
-  (let ((key (if (consp test) (car test) test))
-        (body (if (consp test) (cdr test) nil)))
-    (cond ((null key)
-           nil)
-          ((eq key t)
-           t)
-          ((eq key :constant)
-           (let* ((arg (arg-or-lose subj funstate))
-                  (fields (arg-fields arg))
-                  (consts body))
-             (when (not (= (length fields) (length consts)))
-               (pd-error "The number of constants doesn't match number of ~
-                          fields in: (~S :constant~{ ~S~})"
-                         subj body))
-             (compare-fields-form (gen-arg-forms arg :numeric funstate)
-                                  consts)))
-          ((eq key :positive)
-           `(> ,(arg-value-form (arg-or-lose subj funstate) funstate :numeric)
-               0))
-          ((eq key :negative)
-           `(< ,(arg-value-form (arg-or-lose subj funstate) funstate :numeric)
-               0))
-          ((eq key :test)
-           `(,@body ,(arg-value-form (arg-or-lose subj funstate) funstate :numeric)))
-          ((eq key :same-as)
-           (let ((arg1 (arg-or-lose subj funstate))
-                 (arg2 (arg-or-lose (car body) funstate)))
-             (unless (and (= (length (arg-fields arg1))
-                             (length (arg-fields arg2)))
-                          (every (lambda (bs1 bs2)
-                                   (= (byte-size bs1) (byte-size bs2)))
-                                 (arg-fields arg1)
-                                 (arg-fields arg2)))
-               (pd-error "can't compare differently sized fields: ~
-                          (~S :same-as ~S)" subj (car body)))
-             (compare-fields-form (gen-arg-forms arg1 :numeric funstate)
-                                  (gen-arg-forms arg2 :numeric funstate))))
-          ((eq key :or)
-           `(or ,@(mapcar (lambda (sub) (compile-test subj sub funstate))
-                          body)))
-          ((eq key :and)
-           `(and ,@(mapcar (lambda (sub) (compile-test subj sub funstate))
-                           body)))
-          ((eq key :not)
-           `(not ,(compile-test subj (car body) funstate)))
-          ((and (consp key) (null body))
-           (compile-test subj key funstate))
-          (t
-           (pd-error "bogus test-form: ~S" test)))))
-
 #-sb-fluid (declaim (inline bytes-to-bits))
 
 (defun bytes-to-bits (bytes)
