@@ -41,6 +41,14 @@
   ;; the EQ table.
   (similar-table (make-hash-table :test 'equal) :type hash-table)
   (eq-table (make-hash-table :test 'eq) :type hash-table)
+  ;; the CONS table is an additional EQ table, used for storing CDRs
+  ;; of dumped lists which will not have their own direct identity as
+  ;; a dumped constant, but which might nevertheless be EQ to some
+  ;; other dumped object (and require that EQness to be preserved).
+  ;; The hash table entry, if present, is a reference to its parent
+  ;; list object (which will have a direct entity as a dumped
+  ;; constant) along with an index of how many CDRs to take.
+  (cons-table (make-hash-table :test 'eq) :type hash-table)
   ;; Hashtable mapping a string to a list of fop-table indices of
   ;; symbols whose name is that string. For any name as compared
   ;; by STRING= there can be a symbol whose name is a base string
@@ -629,6 +637,31 @@
         (when (cdr-circularity l n)
           (return))
 
+        ;; if this CONS is EQ to some other object we have already
+        ;; dumped, dump a reference to that instead.
+        (let ((index (gethash l (fasl-output-eq-table file))))
+          (when index
+            (dump-push index file)
+            (terminate-dotted-list n file)
+            (return)))
+
+        ;; if this CONS is EQ to the Ith CDR of some other list we have
+        ;; already dumped, dump a reference to that instead.
+        (let ((list+i (gethash l (fasl-output-cons-table file))))
+          (when list+i
+            (destructuring-bind (list i) list+i
+              (aver (consp list))
+              (let ((index (gethash list (fasl-output-eq-table file))))
+                (dump-push index file)
+                (dump-fop 'fop-nthcdr file i)
+                (when (> n 0)
+                  (terminate-dotted-list n file))
+                (return)))))
+
+        ;; put an entry for this cons into the fasl output cons table,
+        ;; for the benefit of dumping later constants
+        (setf (gethash l (fasl-output-cons-table file)) (list list n))
+
         (setf (gethash l circ) list)
 
         (let* ((obj (car l))
@@ -1070,6 +1103,19 @@
       (dump-push code-handle file)
       (dump-fop 'fop-fun-entry file (decf fun-index))
       (let ((entry-handle (dump-pop file)))
+        ;; When cross compiling, if the entry is a DEFUN, then we also
+        ;; dump a FOP-FSET so that the cold loader can instantiate the
+        ;; definition at cold-load time, allowing forward references
+        ;; to functions in top-level forms.
+        #+sb-xc-host
+        (let ((name (sb-c::entry-info-name entry)))
+          ;; At the moment, we rely on the fopcompiler to do linking
+          ;; for DEFUNs that are not block compiled.
+          (when (and (eq (sb-c::block-compile sb-c::*compilation*) t)
+                     (sb-c::legal-fun-name-p name))
+            (dump-object name file)
+            (dump-push entry-handle file)
+            (dump-fop 'fop-fset file)))
         (setf (gethash entry (fasl-output-entry-table file)) entry-handle)
         (let ((old (gethash entry (fasl-output-patch-table file))))
           (when old
@@ -1165,7 +1211,7 @@
                     (layout-classoid obj)))
   ;; STANDARD-OBJECT could in theory be dumpable, but nothing else,
   ;; because all its subclasses can evolve to have new layouts.
-  (aver (not (logtest (layout-%bits obj) +pcl-object-layout-flag+)))
+  (aver (not (logtest (layout-flags obj) +pcl-object-layout-flag+)))
   (let ((name (layout-classoid-name obj)))
     ;; Q: Shouldn't we aver that NAME is the proper name for its classoid?
     (unless name
@@ -1182,5 +1228,5 @@
   (sub-dump-object (layout-inherits obj) file)
   (dump-fop 'fop-layout file
             (1+ (layout-depthoid obj)) ; non-stack args can't be negative
-            (layout-flags obj)
+            (logand (layout-flags obj) sb-kernel::layout-flags-mask)
             (layout-length obj)))
