@@ -18,8 +18,7 @@
 ;;; An OPAQUE-BOX instance is used to pass data from IR1 to IR2 as
 ;;; a quoted object in a "source form" (not user-written) such that the
 ;;; contained object is in a for-evaluation position but ignored by
-;;; the compiler's constant-dumping logic. In addition to this structure
-;;; type, a few other IR1 object types are implicitly opaque.
+;;; the compiler's constant-dumping logic.
 (defstruct (opaque-box (:constructor opaquely-quote (value))
                        (:copier nil)
                        (:predicate opaque-box-p))
@@ -75,7 +74,7 @@
 ;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
 ;;;             instantiated once per component, INLINE - for all
 ;;;             references (even under #'without FUNCALL)."
-(def!type inlinep ()
+(deftype inlinep ()
   '(member inline maybe-inline notinline nil))
 
 (defstruct (dxable-args (:constructor make-dxable-args (list))
@@ -155,6 +154,12 @@ possible. Potentially long (over one page in size) vectors are, however, not
 stack allocated except in zero SAFETY code, as such a vector could overflow
 the stack without triggering overflow protection.")
 
+;;; *BLOCK-COMPILE-ARGUMENT* holds the original value of the :BLOCK-COMPILE
+;;; argument, which overrides any internal declarations.
+(defvar *block-compile-argument*)
+(declaim (type (member nil t :specified)
+               *block-compile-default* *block-compile-argument*))
+
 ;;; This lock is seized in the compiler, and related areas -- like the
 ;;; classoid/layout/class system.
 ;;; Assigning a literal object enables genesis to dump and load it
@@ -363,6 +368,10 @@ the stack without triggering overflow protection.")
   ;; for constant coalescing across code components, and/or for situations
   ;; where SIMILARP does not do what you want.
   (constant-cache)
+  ;; When compiling within the extent of *macro-policy* we have to store up
+  ;; any DECLAIMs for later replay. The logic is explained in EVAL-COMPILE-TLF.
+  ;; This slot is set to NIL before use and reset when done.
+  (saved-optimize-decls :none)
   (coverage-metadata nil :type (or (cons hash-table hash-table) null) :read-only t)
   (msan-unpoison nil :read-only t)
   (sset-counter 1 :type fixnum)
