@@ -34,26 +34,11 @@
 
 (in-package "SB-IMPL")
 
-#-no-ansi-print-object
 (defmethod print-object ((x meta-info) stream)
   (print-unreadable-object (x stream)
     (format stream "~S ~S, ~D" (meta-info-category x) (meta-info-kind x)
             (meta-info-number x))))
 
-;;; Given any non-negative integer, return a prime number >= to it.
-;;;
-;;; FIXME: This logic should be shared with ALMOST-PRIMIFY in
-;;; hash-table.lisp. Perhaps the merged logic should be
-;;; PRIMIFY-HASH-TABLE-SIZE, implemented as a lookup table of primes
-;;; after integral powers of two:
-;;;    #(17 37 67 131 ..)
-;;; (Or, if that's too coarse, after half-integral powers of two.) By
-;;; thus getting rid of any need for primality testing at runtime, we
-;;; could punt POSITIVE-PRIMEP, too.
-(defun primify (x)
-  (declare (type unsigned-byte x))
-  (do ((n (logior x 1) (+ n 2)))
-      ((positive-primep n) n)))
 
 ;;; Why do we suppress the :COMPILE-TOPLEVEL situation here when we're
 ;;; running the cross-compiler? The cross-compiler (which was built
@@ -93,17 +78,15 @@
         ;; The SB-FASL: symbols are poor style, but the lesser evil.
         ;; If exported, then they'll stick around in the target image.
         ;; Perhaps SB-COLD should re-export some of these.
-        (declare (special sb-fasl::*dynamic* sb-fasl::*cold-layouts*))
-        (let ((layout (gethash 'meta-info sb-fasl::*cold-layouts*)))
-          (sb-fasl::cold-svset
-           (sb-fasl::cold-symbol-value '*info-types*)
-           id
-           (sb-fasl::write-slots
-            (sb-fasl::allocate-struct sb-fasl::*dynamic* layout)
-            'meta-info ; give the type name in lieu of layout
-            :category category :kind kind :type-spec type-spec
-            :type-checker checker :validate-function validator
-            :default default :number id)))))
+        (sb-fasl::cold-svset
+         (sb-fasl::cold-symbol-value '*info-types*)
+         id
+         (sb-fasl::write-slots
+          (sb-fasl::allocate-struct-of-type 'meta-info)
+          'meta-info ; pass the type name in lieu of layout
+          :category category :kind kind :type-spec type-spec
+          :type-checker checker :validate-function validator
+          :default default :number id))))
 
 ;;;; info types, and type numbers, part II: what's
 ;;;; needed only at compile time, not at run time
@@ -182,6 +165,8 @@
   ;; to be certain that it's not supposed to happen when building the xc.
   #+sb-xc-xhost (error "Strange CLEAR-INFO building the xc: ~S ~S"
                        name info-numbers)
+  (when (pcl-methodfn-name-p name)
+    (error "Can't SET-INFO-VALUE on PCL-internal function"))
   (let (new)
     (with-globaldb-name (key1 key2) name
       :simple
@@ -265,8 +250,7 @@
       (values answer nil))))
 
 ;;;; ":FUNCTION" subsection - Data pertaining to globally known functions.
-
-(define-info-type (:function :definition) :type-spec (or #-sb-xc-host fdefn null))
+(define-info-type (:function :definition) :type-spec #-sb-xc-host (or fdefn null) #+sb-xc-host t)
 
 ;;; the kind of functional object being described. If null, NAME isn't
 ;;; a known functional object.
@@ -280,19 +264,23 @@
   ;; 19990330
   :default
   #+sb-xc-host nil
-  #-sb-xc-host (lambda (name) (if (fboundp name) :function nil)))
+  #-sb-xc-host (lambda (name)
+                 (if (or (fboundp name) (pcl-methodfn-name-p name))
+                     :function
+                     nil)))
 
 ;;; The deferred mode processor for fasteval special operators.
 ;;; Immediate processors are hung directly off symbols in a dedicated slot.
-#!+sb-fasteval
+#+sb-fasteval
 (define-info-type (:function :interpreter) :type-spec (or function null))
 
 ;;; Indicates whether the function is deprecated.
 (define-info-type (:function :deprecated)
   :type-spec (or null deprecation-info))
 
-(declaim (ftype (sfunction (t) ctype)
-                specifier-type ctype-of sb-kernel::ctype-of-array))
+;;; Why are these here? It seems like the wrong place.
+(declaim (ftype (sfunction (t &optional t symbol) ctype) specifier-type)
+         (ftype (sfunction (t) ctype) ctype-of sb-kernel::ctype-of-array))
 
 ;;; the ASSUMED-TYPE for this function, if we have to infer the type
 ;;; due to not having a declaration or definition
@@ -357,6 +345,8 @@
 ;;; We don't actually have anything like that any more though.
 ;;; For user-defined functions, the invariant is maintained that at most
 ;;; one of :source-transform and an inline-expansion exist.
+;;; However, there is one exception: a structure constructor can have an
+;;; inline expansion and also store (#<dd> . :constructor) here.
 (define-info-type (:function :source-transform)
   :type-spec (or function null (cons atom atom)))
 
@@ -374,6 +364,9 @@
 ;;; structure containing the info used to special-case compilation.
 (define-info-type (:function :info) :type-spec (or sb-c::fun-info null))
 
+;;; For PCL code walker
+(define-info-type (:function :walker-template) :type-spec (or list symbol))
+
 ;;; This is a type specifier <t> such that if an argument X to the function
 ;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
 ;;; When the named function is a predicate that appears in (SATISFIES p)
@@ -458,7 +451,7 @@
                        ;; The compiler-macro signals an error
                        ;; on forward-referenced info-types.
                        #+sb-xc-host (declare (notinline info))
-                       (when (info :declaration :recognized name)
+                       (when (info :declaration :known name)
                          (error 'declaration-type-conflict-error
                                 :format-arguments (list name)))))
 
@@ -500,8 +493,8 @@
 ;; Therefore maintain a list of recognized declarations. This list makes the
 ;; globaldb storage of same redundant, but oh well.
 (defglobal *recognized-declarations* nil)
-(define-info-type (:declaration :recognized)
-  :type-spec boolean
+(define-info-type (:declaration :known)
+  :type-spec (or function boolean)
   ;; There's no portable way to unproclaim that a symbol is a declaration,
   ;; but at the low-level permit new-value to be NIL.
   :validate-function (lambda (name new-value)
@@ -515,8 +508,6 @@
                               (setq *recognized-declarations*
                                     (delete name *recognized-declarations*))))))
 
-(define-info-type (:declaration :handler) :type-spec (or function null))
-
 ;;;; ":ALIEN-TYPE" subsection - Data pertaining to globally known alien-types.
 (define-info-type (:alien-type :kind)
   :type-spec (member :primitive :defined :unknown)
