@@ -1,9 +1,10 @@
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ; normally assem-scheduler-p is t, and nil if debugging the assembler
 ; but apparently we don't trust any of the instruction definitions.
-(defconstant sb!assem:assem-scheduler-p nil)
-(defconstant sb!assem:+inst-alignment-bytes+ 4)
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+(defconstant sb-assem:+assem-max-locations+ 68) ; see number-location
 
 (defconstant +backend-fasl-file-implementation+ :hppa)
 (defconstant +backend-page-bytes+ 4096)
@@ -26,7 +27,6 @@
 (defconstant single-float-normal-exponent-min 1)
 (defconstant single-float-normal-exponent-max 254)
 (defconstant single-float-hidden-bit (ash 1 23))
-(defconstant single-float-trapping-nan-bit (ash 1 22))
 
 (defconstant double-float-bias 1022)
 (defconstant-eqx double-float-exponent-byte (byte 11 20) #'equalp)
@@ -34,7 +34,6 @@
 (defconstant double-float-normal-exponent-min 1)
 (defconstant double-float-normal-exponent-max #x7FE)
 (defconstant double-float-hidden-bit (ash 1 20))
-(defconstant double-float-trapping-nan-bit (ash 1 19))
 
 (defconstant single-float-digits
   (+ (byte-size single-float-significand-byte) 1))
@@ -66,6 +65,9 @@
 
 ;;; Where to put the different spaces.
 ;;;
+(defconstant linkage-table-space-start #x1000)
+(defconstant linkage-table-space-end   linkage-table-space-start) ; 0 size
+
 (defconstant read-only-space-start #x4b000000)
 (defconstant read-only-space-end   #x4dff0000)
 
