@@ -32,7 +32,6 @@
 ;;; this end, we convert source forms to strings so that source forms
 ;;; that contain IR1 references (e.g. %DEFUN) don't hold onto the IR.
 (defstruct (compiler-error-context
-            #-no-ansi-print-object
             (:print-object (lambda (x stream)
                              (print-unreadable-object (x stream :type t))))
             (:copier nil))
@@ -54,8 +53,9 @@
   initialized
   ;; the original source part of the source path
   (original-source-path nil :type list)
-  ;; the lexenv active at the time
-  (lexenv nil :type (or null lexenv)))
+  ;; lexenv-handled-conditions of the lexenv active at the time
+  (handled-conditions nil))
+(declaim (freeze-type compiler-error-context))
 
 ;;; Delay computing some source information, since it may not actually be ever used
 (defun compiler-error-context-original-source (context)
@@ -265,12 +265,15 @@
                       (nth-value 1 (find-source-root tlf *source-info*))
                       :path path
                       :original-source-path (source-path-original-source path)
-                      :lexenv (cond ((node-p context)
-                                     (node-lexenv context))
-                                    ((lvar-annotation-p context)
-                                     (lvar-annotation-lexenv context))
-                                    ((boundp '*lexenv*)
-                                     *lexenv*)))
+                      :handled-conditions
+                      (let ((lexenv (cond ((node-p context)
+                                           (node-lexenv context))
+                                          ((lvar-annotation-p context)
+                                           (lvar-annotation-lexenv context))
+                                          ((boundp '*lexenv*)
+                                           *lexenv*))))
+                        (and lexenv
+                             (lexenv-handled-conditions lexenv))))
                      nil)))))))))
 
 ;;;; printing error messages
