@@ -449,7 +449,7 @@
 
         ;; Now that the segment is done, convert it to a static
         ;; vector we can point foreign code to.
-        (let* ((buffer (sb-assem::segment-buffer segment))
+        (let* ((buffer (sb-assem:segment-buffer segment))
                (vector (make-static-vector (length buffer)
                                            :element-type '(unsigned-byte 8)
                                            :initial-contents buffer))
