@@ -36,24 +36,70 @@
   (tagify alloc-tn rsp-tn lowtag)
   (values))
 
-(defun %alloc-tramp (node result-tn size lowtag)
-  (cond ((typep size '(and integer (not (signed-byte 32))))
-         ;; MOV accepts large immediate operands, PUSH does not
-         (inst mov result-tn size)
-         (inst push result-tn))
-        (t
-         (inst push size)))
-  ;; This really would be better if it recognized TEMP-REG-TN as the "good" case
-  ;; rather than specializing on R11, which just happens to be the temp reg.
-  ;; But the assembly routine is hand-written, not generated, and it has to match,
-  ;; so there's not much that can be done to generalize it.
-  (let ((to-r11 (location= result-tn r11-tn)))
-    (invoke-asm-routine 'call (if to-r11 'alloc-tramp-r11 'alloc-tramp) node)
+;;; For assemfile
+#+(and avx2 sb-xc-host)
+(defvar *avx-registers-used-p* nil)
+
+#+avx2
+(defun avx-registers-used-p ()
+  (or #+sb-xc-host *avx-registers-used-p*
+      (when (and #+sb-xc-host (boundp '*component-being-compiled*))
+        (let ((comp (component-info *component-being-compiled*)))
+          (or (sb-c::ir2-component-avx2-used-p comp)
+              (flet ((used-p (tn)
+                       (do ((tn tn (sb-c::tn-next tn)))
+                           ((null tn))
+                         (when (sc-is tn avx2-reg
+                                      int-avx2-reg
+                                      double-avx2-reg single-avx2-reg)
+                           (return-from avx-registers-used-p
+                             (setf (sb-c::ir2-component-avx2-used-p comp) t))))))
+                (used-p (sb-c::ir2-component-normal-tns comp))
+                (used-p (sb-c::ir2-component-wired-tns comp))))))))
+
+;;; Call an allocator trampoline and get the result in the proper register.
+;;; There are 2x2x2 choices of trampoline:
+;;;  - invoke alloc() or alloc_list() in C
+;;;  - place result into R11, or leave it on the stack
+;;;  - preserve YMM registers around the call, or don't
+;;; Rather than have 8 different DEFINE-ASSEMBLY-ROUTINEs, there are only 4,
+;;; and each has 2 entry points. The earlier entry adds 1 more instruction
+;;; to set the non-cons bit (the first of the binary choices mentioned above).
+;;; Most of the time, the inline allocation sequence wants to use the trampoline
+;;; that returns a result in TEMP-REG-TN (R11) which saves one move and
+;;; clears the size argument from the stack in the RET instruction.
+;;; If the result is returned on the stack, then we pop it below.
+(defun %alloc-tramp (type node result-tn size lowtag)
+  (let ((consp (eq type 'list))
+        (to-r11 (location= result-tn r11-tn)))
+    (when (typep size 'integer)
+      (aver (= (align-up size (* 2 n-word-bytes)) size))
+      (when (neq type 'list)
+        (incf size) ; the low bit means we're allocating a non-cons object
+        ;; Jump into the cons entry point which saves one instruction because why not.
+        (setq consp t)))
+    (cond ((typep size '(and integer (not (signed-byte 32))))
+           ;; MOV accepts large immediate operands, PUSH does not
+           (inst mov result-tn size)
+           (inst push result-tn))
+          (t
+           (inst push size)))
+    (invoke-asm-routine
+     'call
+     (cond #+avx2
+           ((avx-registers-used-p)
+            (if to-r11
+                (if consp 'cons->r11.avx2 'alloc->r11.avx2)
+                (if consp 'cons->rnn.avx2 'alloc->rnn.avx2)))
+           (t
+            (if to-r11
+                (if consp 'cons->r11 'alloc->r11)
+                (if consp 'cons->rnn 'alloc->rnn))))
+     node)
     (unless to-r11
       (inst pop result-tn)))
   (unless (eql lowtag 0)
-    (inst or :byte result-tn lowtag))
-  (values))
+    (inst or :byte result-tn lowtag)))
 
 ;;; Insert allocation profiler instrumentation
 (defun instrument-alloc (size node)
@@ -91,39 +137,38 @@
 ;;; NODE may be used to make policy-based decisions.
 ;;; This function should only be used inside a pseudo-atomic section,
 ;;; which to the degree needed should also cover subsequent initialization.
-(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
+;;; CONSP says whether we're allocating conses. But we also need LOWTAG
+;;; because the vop could want the CONS to have 0 or list-pointer lowtag.
+;;;
+;;; A mnemonic device for the argument pattern here:
+;;; 1. what to allocate: type, size, lowtag describe the object
+;;; 2. how to allocate it: policy and how to invoke the trampoline
+;;; 3. where to put the result
+(defun allocation (type size lowtag node dynamic-extent alloc-tn)
   (when dynamic-extent
     (stack-allocation alloc-tn size lowtag)
     (return-from allocation (values)))
   (aver (and (not (location= alloc-tn temp-reg-tn))
              (or (integerp size) (not (location= size temp-reg-tn)))))
 
-  #+(and (not sb-thread) sb-dynamic-core)
-  ;; We'd need a spare reg in which to load boxed_region from the linkage table.
-  ;; Could push/pop any random register on the stack and own it temporarily,
-  ;; but seeing as nobody cared about this, just punt.
-  (%alloc-tramp node alloc-tn size lowtag)
-
-  #-(and (not sb-thread) sb-dynamic-core)
+  (aver (not (sb-assem::assembling-to-elsewhere-p)))
   ;; Otherwise do the normal inline allocation thing
   (let ((NOT-INLINE (gen-label))
         (DONE (gen-label))
-        ;; Yuck.
-        (in-elsewhere (sb-assem::assembling-to-elsewhere-p))
         ;; thread->alloc_region.free_pointer
         (free-pointer
          #+sb-thread (thread-slot-ea thread-alloc-region-slot)
-         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign)))
+         #-sb-thread (ea (+ static-space-start
+                            (ash vector-data-offset word-shift))))
         ;; thread->alloc_region.end_addr
         (end-addr
          #+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
-         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign 8))))
+         #-sb-thread (ea (+ static-space-start
+                            (ash (1+ vector-data-offset) word-shift)))))
 
-    (cond ((or in-elsewhere
-               ;; large objects will never be made in a per-thread region
-               (and (integerp size)
-                    (>= size large-object-size)))
-           (%alloc-tramp node alloc-tn size lowtag))
+    (cond ((typep size `(integer , large-object-size))
+           ;; large objects will never be made in a per-thread region
+           (%alloc-tramp type node alloc-tn size lowtag))
           ((eql lowtag 0)
            (cond ((and (tn-p size) (location= size alloc-tn))
                   (inst mov temp-reg-tn free-pointer)
@@ -143,7 +188,7 @@
                                 temp-reg-tn)
                                (t
                                 size))))
-               (%alloc-tramp node alloc-tn size 0))
+               (%alloc-tramp type node alloc-tn size 0))
              (inst jmp DONE)))
           (t
            (inst mov temp-reg-tn free-pointer)
@@ -161,10 +206,10 @@
            (assemble (:elsewhere)
              (emit-label NOT-INLINE)
              (cond ((and (tn-p size) (location= size alloc-tn)) ; recover SIZE
-                    (inst sub alloc-tn free-pointer)
-                    (%alloc-tramp node temp-reg-tn alloc-tn 0))
+                    (inst sub alloc-tn temp-reg-tn)
+                    (%alloc-tramp type node temp-reg-tn alloc-tn 0))
                    (t ; SIZE is intact
-                    (%alloc-tramp node temp-reg-tn size 0)))
+                    (%alloc-tramp type node temp-reg-tn size 0)))
              (inst jmp DONE))))
     (values)))
 
@@ -173,14 +218,14 @@
 ;;; RESULT-TN.
 (defun alloc-other (result-tn widetag size node &optional stack-allocate-p
                     &aux (bytes (pad-data-block size)))
-  (let ((header (logior (ash (1- size) n-widetag-bits) widetag)))
+  (let ((header (compute-object-header size widetag)))
     (cond (stack-allocate-p
-           (allocation result-tn bytes node t other-pointer-lowtag)
+           (allocation nil bytes other-pointer-lowtag node t result-tn)
            (storew header result-tn 0 other-pointer-lowtag))
           (t
            (instrument-alloc bytes node)
            (pseudo-atomic ()
-             (allocation result-tn bytes node nil 0)
+             (allocation nil bytes 0 node nil result-tn)
              (storew* header result-tn 0 0 t)
              (inst or :byte result-tn other-pointer-lowtag))))))
 
@@ -220,8 +265,8 @@
                (unless stack-allocate-p
                  (instrument-alloc size node))
                (pseudo-atomic (:elide-if stack-allocate-p)
-                (allocation res size node stack-allocate-p
-                            (if (= cons-cells 2) 0 list-pointer-lowtag))
+                (allocation 'list size (if (= cons-cells 2) 0 list-pointer-lowtag)
+                            node stack-allocate-p res)
                 (multiple-value-bind (last-base-reg lowtag car cdr)
                     (cond
                       ((= cons-cells 2)
@@ -329,7 +374,7 @@
       (let ((size (calc-size-in-bytes words result)))
         (instrument-alloc size node)
         (pseudo-atomic ()
-         (allocation result size node nil 0)
+         (allocation nil size 0 node nil result)
          (put-header result 0 type length t)
          (inst or :byte result other-pointer-lowtag)))))
 
@@ -479,7 +524,7 @@
              (and (sc-is element immediate) (eql (tn-value element) 0))))
         (instrument-alloc size node)
         (pseudo-atomic ()
-         (allocation result size node nil list-pointer-lowtag)
+         (allocation 'list size list-pointer-lowtag node nil result)
          (compute-end)
          (inst mov next result)
          (inst jmp entry)
@@ -522,7 +567,7 @@
      (unless stack-allocate-p
        (instrument-alloc bytes node))
      (pseudo-atomic (:elide-if stack-allocate-p)
-       (allocation result bytes node stack-allocate-p fun-pointer-lowtag)
+       (allocation nil bytes fun-pointer-lowtag node stack-allocate-p result)
        (storew* #-immobile-space header ; write the widetag and size
                 #+immobile-space        ; ... plus the layout pointer
                 (progn (inst mov temp header)
@@ -532,8 +577,9 @@
                        temp)
                 result 0 fun-pointer-lowtag (not stack-allocate-p)))
      ;; Done with pseudo-atomic
-     (inst lea temp (rip-relative-ea label (ash simple-fun-code-offset word-shift)))
-     (storew temp result closure-fun-slot fun-pointer-lowtag))))
+     (when label
+       (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
+       (storew temp result closure-fun-slot fun-pointer-lowtag)))))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -583,10 +629,10 @@
            (t
             ;; If storing a header word, defer ORing in the lowtag until after
             ;; the header is written so that displacement can be 0.
-            (allocation result bytes node stack-allocate-p (if type 0 lowtag))
+            (allocation nil bytes (if type 0 lowtag) node stack-allocate-p result)
             (when type
               (let* ((widetag (if instancep instance-widetag type))
-                     (header (logior (ash (1- words) n-widetag-bits) widetag)))
+                     (header (logior (ash (1- words) (length-field-shift widetag)) widetag)))
                 (if (or #+compact-instance-header
                         (and (eq name '%make-structure-instance) stack-allocate-p))
                     ;; Write a :DWORD, not a :QWORD, because the high half will be
@@ -604,14 +650,14 @@
 ;;; Exactly 4 allocators are rendered via this vop:
 ;;;  BIGNUM               (%ALLOCATE-BIGNUM)
 ;;;  FUNCALLABLE-INSTANCE (%MAKE-FUNCALLABLE-INSTANCE)
-;;;  CLOSURE              (%COPY-CLOSURE)
+;;;  CLOSURE              (%ALLOC-CLOSURE)
 ;;;  INSTANCE             (%MAKE-INSTANCE)
 ;;; WORDS accounts for the mandatory slots *including* the header.
 ;;; EXTRA is the variable payload, also measured in words.
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
-  (:info name words type lowtag)
+  (:info name words type lowtag stack-allocate-p)
   (:ignore name)
   (:results (result :scs (descriptor-reg) :from (:eval 1)))
   (:temporary (:sc unsigned-reg :from :eval :to (:eval 1)) bytes)
@@ -625,14 +671,18 @@
             (ea (* (1+ words) n-word-bytes) nil
                 extra (ash 1 (- word-shift n-fixnum-tag-bits))))
       (inst mov operand-size header bytes)
-      (inst shl operand-size header (- n-widetag-bits word-shift)) ; w+1 to length field
+      (inst shl operand-size header (- (length-field-shift type) word-shift)) ; w+1 to length field
       (inst lea operand-size header                    ; (w-1 << 8) | type
-            (ea (+ (ash -2 n-widetag-bits) type) header))
+            (ea (+ (ash -2 (length-field-shift type)) type) header))
       (inst and operand-size bytes (lognot lowtag-mask)))
-      (instrument-alloc bytes node)
-      (pseudo-atomic ()
-       (allocation result bytes node nil lowtag)
-       (storew header result 0 lowtag))))
+      (cond (stack-allocate-p
+             (stack-allocation result bytes lowtag)
+             (storew header result 0 lowtag))
+            (t
+             (instrument-alloc bytes node)
+             (pseudo-atomic ()
+              (allocation nil bytes lowtag node nil result)
+              (storew header result 0 lowtag))))))
 
 (macrolet ((c-call (name)
              `(let ((c-fun (make-fixup ,name :foreign)))
@@ -671,7 +721,7 @@
    ;; which has that effect
    (inst and rsp-tn -16)
    (pseudo-atomic () (c-call "alloc_code_object"))
-   ;; C-RESULT is a tagged ptr. MOV doesn't need to be pseudo-atomic.
+   ;; C-RESULT is a tagged ptr. MOV doesn't need to be inside the PSEUDO-ATOMIC.
    (inst mov result c-result)))
 
 ) ; end MACROLET
