@@ -11,10 +11,10 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-(defconstant sb!assem:assem-scheduler-p nil)
-(defconstant sb!assem:+inst-alignment-bytes+ 1)
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 1)
 
 (defconstant +backend-fasl-file-implementation+ :x86-64)
 (defconstant-eqx +fixup-kinds+ #(:absolute :relative :absolute64)
@@ -29,7 +29,7 @@
 ;;; useless in SBCL, since it's possible for otherwise binary
 ;;; compatible systems to return different values for getpagesize().
 ;;; -- JES, 2007-01-06
-(defconstant +backend-page-bytes+ #!+win32 65536 #!-win32 32768)
+(defconstant +backend-page-bytes+ #+win32 65536 #-win32 32768)
 
 ;;; The size in bytes of GENCGC cards, i.e. the granularity at which
 ;;; writes to old generations are logged.  With mprotect-based write
@@ -64,12 +64,6 @@
 ;;; address space)
 (defconstant n-machine-word-bits 64)
 
-;;; The minimum immediate offset in a memory-referencing instruction.
-(defconstant minimum-immediate-offset (- (expt 2 31)))
-
-;;; The maximum immediate offset in a memory-referencing instruction.
-(defconstant maximum-immediate-offset (1- (expt 2 31)))
-
 (defconstant float-sign-shift 31)
 
 ;;; comment from CMU CL:
@@ -85,7 +79,6 @@
 (defconstant single-float-normal-exponent-min 1)
 (defconstant single-float-normal-exponent-max 254)
 (defconstant single-float-hidden-bit (ash 1 23))
-(defconstant single-float-trapping-nan-bit (ash 1 22))
 
 (defconstant double-float-bias 1022)
 (defconstant-eqx double-float-exponent-byte    (byte 11 20) #'equalp)
@@ -93,7 +86,6 @@
 (defconstant double-float-normal-exponent-min 1)
 (defconstant double-float-normal-exponent-max #x7FE)
 (defconstant double-float-hidden-bit (ash 1 20))
-(defconstant double-float-trapping-nan-bit (ash 1 19))
 
 (defconstant single-float-digits
   (+ (byte-size single-float-significand-byte) 1))
@@ -132,8 +124,9 @@
 ;;; would be possible, but probably not worth the time and code bloat
 ;;; it would cause. -- JES, 2005-12-11
 
-#!+linux
+#+linux
 (!gencgc-space-setup #x50000000
+                     :read-only-space-size 0
                      :fixedobj-space-size #.(* 30 1024 1024)
                      :varyobj-space-size #.(* 130 1024 1024)
                      :dynamic-space-start #x1000000000)
@@ -141,11 +134,13 @@
 ;;; The default dynamic space size is lower on OpenBSD to allow SBCL to
 ;;; run under the default 512M data size limit.
 
-#!-linux
+#-linux
 (!gencgc-space-setup #x20000000
+                     #-win32 :read-only-space-size #-win32 0
                      :dynamic-space-start #x1000000000
-                     #!+openbsd :dynamic-space-size #!+openbsd #x1bcf0000)
+                     #+openbsd :dynamic-space-size #+openbsd #x1bcf0000)
 
+(defconstant linkage-table-growth-direction :up)
 (defconstant linkage-table-entry-size 16)
 
 
@@ -157,10 +152,14 @@
   fun-end-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
+  undefined-function-trap
   invalid-arg-count-trap
   memory-fault-emulation-trap
-  #!+sb-safepoint global-safepoint-trap
-  #!+sb-safepoint csp-safepoint-trap
+  #+sb-safepoint global-safepoint-trap
+  #+sb-safepoint csp-safepoint-trap
+  uninitialized-load-trap
+  ;; ERROR-TRAP has to be numerically highest.
+  ;; The various internal errors are numbered from here upward.
   error-trap)
 
 ;;;; static symbols
@@ -180,16 +179,14 @@
 
 (defconstant-eqx +static-symbols+
  `#(,@+common-static-symbols+
-    #!+(and immobile-space (not sb-thread)) function-layout
-    #!-sb-thread *alien-stack-pointer*    ; a thread slot if #!+sb-thread
+    #+(and immobile-space (not sb-thread)) function-layout
+    #-sb-thread *alien-stack-pointer*    ; a thread slot if #+sb-thread
      ;; interrupt handling
-    #!-sb-thread *pseudo-atomic-bits*     ; ditto
-    #!-sb-thread *binding-stack-pointer* ; ditto
+    #-sb-thread *pseudo-atomic-bits*     ; ditto
+    #-sb-thread *binding-stack-pointer* ; ditto
     *cpuid-fn1-ecx*)
   #'equalp)
 
-;;; FIXME: with #!+immobile-space, this should be the empty list,
-;;; because *all* fdefns are permanently placed.
 (defconstant-eqx +static-fdefns+
   #(length
     two-arg-+
@@ -206,8 +203,21 @@
     two-arg-xor
     two-arg-gcd
     two-arg-lcm
+    ensure-symbol-hash
+    update-object-layout-or-invalid
     %coerce-callable-to-fun)
   #'equalp)
 
-#!+sb-simd-pack
+#+sb-simd-pack
 (defglobal *simd-pack-element-types* '(integer single-float double-float))
+
+(defconstant undefined-fdefn-header
+  ;; This constant is constructed as follows: Take the INT opcode
+  ;; plus the undefined-fun trap byte, then the bytes of the 'disp' field
+  ;; of the JMP instruction that would overwrite the INT instruction.
+  ;;   INT3 <trap-code> = CC **
+  ;;   JMP [RIP+16]     = FF 25 10 00 00 00 00
+  ;; When assigning a function we'll change the first two bytes to 0xFF 0x25.
+  ;; The 'disp' field will aready be correct.
+  (logior (ash undefined-function-trap 8)
+          (+ #x100000 (or #+int4-breakpoints #xCE #xCC))))
