@@ -133,8 +133,6 @@
   :metaclass-constructor make-static-classoid
   :dd-type funcallable-structure)
 
-(declaim (freeze-type ctor))
-
 ;;; All defined ctors.
 (defglobal *all-ctors* (make-hash-table :test #'equal
                                         :weakness :value))
@@ -239,17 +237,8 @@
 (declaim (inline sxhash-symbol-or-class))
 (defun sxhash-symbol-or-class (x)
   (cond ((symbolp x) (sxhash x))
-        ;; FIXME: if we could ensure that metaobjects (or at least just CLASS
-        ;; metaobjects) always have a precomputed hash, then instead of calling
-        ;; STD-INSTANCE-HASH which might have to compute and install the hash,
-        ;; a faster variant of this can assume that the hash is definitely nonzero.
-        ;; We could go back to always computing hashes for all standard instances,
-        ;; but I don't like that, because it will cause great pain for trying
-        ;; to generate deterministic core images. Since GFs usually have names
-        ;; (and classes do too), but instances in general don't, if you stay away
-        ;; from anonymous objects, reproducibility is an attainable goal.
-        ((std-instance-p x) (sb-impl::std-instance-hash x))
-        ((fsc-instance-p x) (sb-impl::fsc-instance-hash x))
+        ((std-instance-p x) (sb-impl::instance-sxhash x))
+        ((fsc-instance-p x) (fsc-instance-hash x))
         (t
          (bug "Something strange where symbol or class expected."))))
 
@@ -789,11 +778,10 @@
 ;;; the instance, and .SLOTS. to the instance's slot vector around BODY.
 (defun wrap-in-allocate-forms (ctor body early-unbound-markers-p)
   (let* ((class (ctor-class ctor))
-         (wrapper (class-wrapper class))
-         (allocation-function (raw-instance-allocator class))
-         (slots-fetcher (slots-fetcher class)))
-    (if (eq allocation-function 'allocate-standard-instance)
-        `(let ((.instance. (%make-standard-instance nil #-compact-instance-header 0))
+         (wrapper (class-wrapper class)))
+    (etypecase class
+      (standard-class
+        `(let ((.instance. (%make-instance (1+ sb-vm:instance-data-start)))
                (.slots. (make-array
                          ,(layout-length wrapper)
                          ,@(when early-unbound-markers-p
@@ -801,12 +789,10 @@
            (setf (%instance-layout .instance.) ,wrapper)
            (setf (std-instance-slots .instance.) .slots.)
            ,body
-           .instance.)
-        (let ((more
-               (when (member allocation-function '(allocate-standard-funcallable-instance))
-                 '(nil))))
-          `(let* ((.instance. (,allocation-function ,wrapper ,@more))
-                  (.slots. (,slots-fetcher .instance.)))
+           .instance.))
+      (funcallable-standard-class
+        `(let* ((.instance. (allocate-standard-funcallable-instance ,wrapper nil))
+                (.slots. (fsc-instance-slots .instance.)))
              (declare (ignorable .slots.))
              ,body
              .instance.)))))
