@@ -50,6 +50,7 @@
 #include "forwarding-ptr.h"
 #include "var-io.h"
 #include "search.h"
+#include "murmur_hash.h"
 
 #ifdef LISP_FEATURE_SPARC
 #define LONG_FLOAT_SIZE 4
@@ -92,7 +93,15 @@ copy_object(lispobj object, sword_t nwords)
     return gc_general_copy_object(object, nwords, BOXED_PAGE_FLAG);
 }
 
+#ifdef LISP_FEATURE_PPC64
+// unevenly spaced pointer lowtags
+static void (*scav_ptr[16])(lispobj *where, lispobj object); /* forward decl */
+#define PTR_SCAVTAB_INDEX(ptr) (ptr & 15)
+#else
+// evenly spaced pointer lowtags
 static void (*scav_ptr[4])(lispobj *where, lispobj object); /* forward decl */
+#define PTR_SCAVTAB_INDEX(ptr) ((uint32_t)ptr>>(N_LOWTAG_BITS-2))&3
+#endif
 
 static inline void scav1(lispobj* object_ptr, lispobj object)
 {
@@ -122,7 +131,7 @@ static inline void scav1(lispobj* object_ptr, lispobj object)
     if (forwarding_pointer_p(ptr)) \
         *object_ptr = LOW_WORD(forwarding_pointer_value(ptr)); \
     else /* Scavenge that pointer. */ \
-        scav_ptr[(object>>(N_LOWTAG_BITS-2))&3](object_ptr, object); \
+        scav_ptr[PTR_SCAVTAB_INDEX(object)](object_ptr, object);  \
     }
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     page_index_t page;
@@ -428,7 +437,43 @@ static inline lispobj copy_instance(lispobj object)
 {
     // Object is an un-forwarded object in from_space
     lispobj header = *(lispobj*)(object - INSTANCE_POINTER_LOWTAG);
-    lispobj copy = copy_object(object, 1 + (instance_length(header)|1));
+    int original_length = instance_length(header);
+    lispobj copy;
+    // KLUDGE: reading both flags at once doesn't really work
+    // unless either we know what the opaque values are:
+    //  8 = stable-hash-required-flag
+    //  9 = hash-slot-present-flag
+    // Maybe the C compiler is sufficiently smart to turn a boolean expression
+    // involving "&&" into this simple expression?
+    if (((header >> 8) & 3) == 1) { // state = hashed + not moved
+        /* If odd, add 1 (making even). Rounding that to odd and then
+         * adding 1 for the header will effectively add 2 words.
+         * Otherwise, don't add anything because a padding slot exists */
+        int new_length = original_length + (original_length & 1);
+        copy = gc_copy_object_resizing(object, 1 + (new_length|1), BOXED_PAGE_FLAG,
+                                       1 + (original_length|1));
+        lispobj *base = native_pointer(copy);
+        /* store the old address as the hash value */
+#ifdef LISP_FEATURE_64_BIT
+        uint64_t hash = murmur3_fmix64(object);
+#else
+        uint32_t hash = murmur3_fmix32(object);
+#endif
+        hash = (hash << (N_FIXNUM_TAG_BITS+1)) >> 1;
+        base[original_length+1] = hash;
+        /* If the object was enlarged by 2 words, then clear the final word.
+         * Git rev 5f435a2b66 removed prezeroizing during GC, and that
+         * last word was not the target of a memcpy */
+        if (original_length & 1) base[original_length+2] = 0;
+        *base |= 1 << flag_HashSlotPresent;
+#if 0
+        printf("stable hash: obj=%p L=%d -> %p L=%d (recalc=%d)\n",
+               (void*)object, original_length, (void*)copy, new_length,
+               instance_length(*base));
+#endif
+    } else {
+        copy = copy_object(object, 1 + (original_length|1));
+    }
     set_forwarding_pointer(native_pointer(object), copy);
     return copy;
 }
@@ -657,21 +702,18 @@ instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
 static sword_t
 scav_instance(lispobj *where, lispobj header)
 {
-    lispobj lbitmap = make_fixnum(-1);
-    lispobj *layout = 0;
+    int nslots = instance_length(header) | 1;
+    if (!instance_layout(where)) return 1 + nslots;
 
-    if (instance_layout(where)) {
-        layout = native_pointer(instance_layout(where));
+    lispobj *layout = native_pointer(instance_layout(where));
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-        if (immobile_obj_gen_bits(layout) == from_space)
-            enliven_immobile_obj(layout, 1);
+    if (immobile_obj_gen_bits(layout) == from_space)
+        enliven_immobile_obj(layout, 1);
 #else
-        if (forwarding_pointer_p(layout))
-            layout = native_pointer(forwarding_pointer_value(layout));
+    if (forwarding_pointer_p(layout))
+        layout = native_pointer(forwarding_pointer_value(layout));
 #endif
-        lbitmap = ((struct layout*)layout)->bitmap;
-    }
-    sword_t nslots = instance_length(header) | 1;
+    lispobj lbitmap = ((struct layout*)layout)->bitmap;
     if (lbitmap == make_fixnum(-1)) {
         scavenge(where+1, nslots);
         return 1 + nslots;
@@ -702,7 +744,7 @@ scav_instance(lispobj *where, lispobj header)
                       where+1, nslots, lbitmap, 0);
     } else {
         sword_t bitmap = fixnum_value(lbitmap); // signed integer!
-        sword_t n = nslots;
+        int n = nslots;
         lispobj obj;
         for ( ; n-- ; bitmap >>= 1) {
             ++where;
@@ -713,16 +755,34 @@ scav_instance(lispobj *where, lispobj header)
     return 1 + nslots;
 }
 
+static sword_t size_instance(lispobj *where) {
+    return 1 + (instance_length(*where) | 1);
+}
+
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
 static sword_t
 scav_funinstance(lispobj *where, lispobj header)
 {
-    // This works because the layout is in the header word of all instances,
-    // ordinary and funcallable, when compact headers are enabled.
-    // The trampoline slot in the funcallable-instance is raw, but can be
-    // scavenged, because it points to readonly space, never oldspace.
-    // (And for certain backends it looks like a fixnum, not a pointer)
-    return scav_instance(where, header);
+    // Do a similar thing as scav_instance but do not split into 3 cases
+    // based on whether the bitmap is a fixnum or a bignum or the special
+    // case of all tagged; it's always a fixnum, with at least 1 raw slot.
+    int nslots = SHORT_BOXED_NWORDS(header);
+    if (!instance_layout(where)) return 1 + nslots;
+
+    lispobj *layout = native_pointer(instance_layout(where));
+    if (immobile_obj_gen_bits(layout) == from_space)
+        enliven_immobile_obj(layout, 1);
+    lispobj lbitmap = ((struct layout*)layout)->bitmap;
+    gc_assert(fixnump(lbitmap));
+    sword_t bitmap = fixnum_value(lbitmap);
+    int n = nslots;
+    lispobj obj;
+    for ( ; n-- ; bitmap >>= 1) {
+        ++where;
+        if ((bitmap & 1) && is_lisp_pointer(obj = *where))
+            scav1(where, obj);
+    }
+    return 1 + nslots;
 }
 #endif
 
@@ -1181,7 +1241,7 @@ void weakobj_init()
     unsigned i;                                                                \
     for (i = 1; i <= hwm; i++) {                                               \
         lispobj key = data[2*i], value = data[2*i+1];                          \
-        if (is_lisp_pointer(key|value)) {                                      \
+        if (at_least_one_pointer_p(key,value)) {                               \
           if (!entry_alivep) { defer; any_deferred = 1; } else {               \
             /* Scavenge the key and value. */                                  \
             scav_entry(&data[2*i]);                                            \
