@@ -23,6 +23,7 @@
 #include "sbcl.h"
 #include "print.h"
 #include "runtime.h"
+#include "code.h"
 #include "gc-internal.h"
 #include <stdarg.h>
 #include "thread.h"              /* genesis/primitive-objects.h needs this */
@@ -326,7 +327,7 @@ static void brief_otherimm(lispobj obj)
     int type, c;
     char * charname = 0;
 
-    type = widetag_of(obj);
+    type = header_widetag(obj);
     switch (type) {
         case CHARACTER_WIDETAG:
             c = obj>>8; // no mask. show whatever's there
@@ -356,9 +357,9 @@ static void brief_otherimm(lispobj obj)
 
 static void print_otherimm(lispobj obj)
 {
-    printf(", %s", widetag_names[widetag_of(obj) >> 2]);
+    printf(", %s", widetag_names[header_widetag(obj) >> 2]);
 
-    switch (widetag_of(obj)) {
+    switch (header_widetag(obj)) {
     case CHARACTER_WIDETAG:
         printf(": ");
         brief_otherimm(obj);
@@ -418,7 +419,7 @@ static void print_list(lispobj obj)
 // takes native pointer as input
 char * simple_base_stringize(struct vector * string)
 {
-  if (widetag_of(string->header) == SIMPLE_BASE_STRING_WIDETAG)
+  if (widetag_of(&string->header) == SIMPLE_BASE_STRING_WIDETAG)
       return (char*)string->data;
   int length = string->length;
   char * newstring = malloc(length+1);
@@ -462,7 +463,7 @@ static boolean tagged_slot_p(struct layout *layout, int slot_index)
 static void print_struct(lispobj obj)
 {
     struct instance *instance = (struct instance *)native_pointer(obj);
-    unsigned int i;
+    short int i;
     char buffer[16];
     lispobj layout = instance_layout(native_pointer(obj));
     print_obj("type: ", layout);
@@ -483,7 +484,7 @@ void show_lstring(struct vector * string, int quotes, FILE *s)
   int i, len = fixnum_value(string->length);
 
 #ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-  if (widetag_of(string->header) == SIMPLE_CHARACTER_STRING_WIDETAG) {
+  if (widetag_of(&string->header) == SIMPLE_CHARACTER_STRING_WIDETAG) {
       ucs4_p = 1;
       if (quotes)
           putc('u', s); /* an arbitrary notational convention */
@@ -510,7 +511,7 @@ void show_lstring(struct vector * string, int quotes, FILE *s)
   if (quotes) putc('"', s);
 }
 
-static void brief_otherptr(lispobj obj)
+static void brief_fun_or_otherptr(lispobj obj)
 {
     extern void safely_show_lstring(struct vector*, int, FILE*);
     lispobj *ptr, header;
@@ -519,7 +520,7 @@ static void brief_otherptr(lispobj obj)
 
     ptr = native_pointer(obj);
     header = *ptr;
-    type = widetag_of(header);
+    type = header_widetag(header);
     switch (type) {
         case SYMBOL_WIDETAG:
             symbol = (struct symbol *)ptr;
@@ -542,7 +543,7 @@ static void brief_otherptr(lispobj obj)
                 // FIXME: more address validity checks perhaps?
                 lispobj name = ((struct fdefn*)ptr)->name;
                 if (lowtag_of(name) == OTHER_POINTER_LOWTAG
-                    && widetag_of(*native_pointer(name)) == SYMBOL_WIDETAG) {
+                    && widetag_of(native_pointer(name)) == SYMBOL_WIDETAG) {
                   printf(" for ");
                   struct vector* str = symbol_name(native_pointer(name));
                   safely_show_lstring(str, 0, stdout);
@@ -556,14 +557,19 @@ static void print_slots(char **slots, int count, lispobj *ptr)
 {
     while (count-- > 0) {
         if (*slots) {
-            print_obj(*slots++, *ptr++);
+            // kludge for half-lispword sized slot
+            print_obj(*slots,
+                      (N_WORD_BYTES == 8 && !strcmp(*slots, "boxed_size: "))
+                      ? *ptr & 0xFFFFFFFF : *ptr);
+            slots++;
         } else {
-            print_obj("???: ", *ptr++);
+            print_obj("???: ", *ptr);
         }
+        ptr++;
     }
 }
 
-static lispobj symbol_function(lispobj* symbol)
+lispobj symbol_function(lispobj* symbol)
 {
     lispobj info = ((struct symbol*)symbol)->info;
     if (listp(info))
@@ -583,16 +589,14 @@ static lispobj symbol_function(lispobj* symbol)
     return NIL;
 }
 
-static void print_otherptr(lispobj obj)
+static void print_fun_or_otherptr(lispobj obj)
 {
 #ifndef LISP_FEATURE_ALPHA
     lispobj *ptr;
     unsigned long header;
-    unsigned long length;
 #else
     u32 *ptr;
     u32 header;
-    u32 length;
 #endif
     int count, type, index;
     char buffer[16];
@@ -604,9 +608,8 @@ static void print_otherptr(lispobj obj)
     }
 
     header = *ptr++;
-    length = fixnum_value(*ptr);
     count = HeaderValue(header);
-    type = widetag_of(header);
+    type = header_widetag(header);
 
     print_obj("header: ", header);
     if (!other_immediate_lowtag_p(header)) {
@@ -615,8 +618,6 @@ static void print_otherptr(lispobj obj)
         return;
     }
 
-    if (unprintable_array_types[type/8] & (1<<(type % 8)))
-        return;
     switch (type) {
     case BIGNUM_WIDETAG:
         ptr += count;
@@ -711,6 +712,8 @@ static void print_otherptr(lispobj obj)
 
     case SIMPLE_VECTOR_WIDETAG:
         NEWLINE_OR_RETURN;
+        {
+        long length = fixnum_value(*ptr);
         printf("length = %ld", length);
         ptr++;
         index = 0;
@@ -718,12 +721,13 @@ static void print_otherptr(lispobj obj)
             sprintf(buffer, "%d: ", index++);
             print_obj(buffer, *ptr++);
         }
+        }
         break;
 
     // FIXME: This case looks unreachable. print_struct() does it
     case INSTANCE_WIDETAG:
         NEWLINE_OR_RETURN;
-        count &= SHORT_HEADER_MAX_WORDS;
+        count = instance_length(header);
         printf("length = %ld", (long) count);
         index = 0;
         while (count-- > 0) {
@@ -734,7 +738,7 @@ static void print_otherptr(lispobj obj)
 
     case CODE_HEADER_WIDETAG:
         // ptr was already bumped up
-        count = code_header_words(((struct code*)(ptr-1))->header);
+        count = code_header_words((struct code*)(ptr-1));
         for_each_simple_fun(fun_index, fun, (struct code*)(ptr-1), 0, {
             sprintf(buffer, "f[%d]: ", fun_index);
             print_obj(buffer, make_lispobj(fun,FUN_POINTER_LOWTAG));
@@ -743,10 +747,7 @@ static void print_otherptr(lispobj obj)
         break;
 
     case SIMPLE_FUN_WIDETAG:
-        print_obj("code: ",
-                  make_lispobj(native_pointer((lispobj)(ptr-1))
-                               -(HeaderValue(header)&0xFFFF),
-                               OTHER_POINTER_LOWTAG));
+        print_obj("code: ", fun_code_tagged(ptr-1));
         print_slots(simple_fun_slots,
                     sizeof simple_fun_slots/sizeof(char*)-1, ptr);
         break;
@@ -795,34 +796,35 @@ static void print_otherptr(lispobj obj)
         print_obj("entry: ", fdefn_callee_lispobj((struct fdefn*)(ptr-1)));
         break;
 
+    // Make certain vectors printable from C for when all hell breaks lose
+    case SIMPLE_ARRAY_UNSIGNED_BYTE_32_WIDETAG:
+        NEWLINE_OR_RETURN;
+        {
+        long length = fixnum_value(*ptr);
+        uint32_t * data = (uint32_t*)(ptr + 1);
+        long i;
+        printf("#(");
+        for (i=0; i<length; ++i) {
+            printf("%s%d", i>0?" ":"", data[i]);
+            if(i==255 && length>256) { printf(" ..."); break; }
+        }
+        printf(")");
+        }
+        break;
     default:
         NEWLINE_OR_RETURN;
-        printf("Unknown header object?");
+        if (type >= SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG &&
+            type <= SIMPLE_BIT_VECTOR_WIDETAG)
+            printf("length = %ld", (long)fixnum_value(*ptr));
+        else
+            printf("Unknown header object?");
         break;
     }
 }
 
 static void print_obj(char *prefix, lispobj obj)
 {
-#ifdef LISP_FEATURE_64_BIT
-    static void (*verbose_fns[])(lispobj obj)
-        = {print_fixnum, print_otherimm, print_fixnum, print_struct,
-           print_fixnum, print_otherimm, print_fixnum, print_list,
-           print_fixnum, print_otherimm, print_fixnum, print_otherptr,
-           print_fixnum, print_otherimm, print_fixnum, print_otherptr};
-    static void (*brief_fns[])(lispobj obj)
-        = {brief_fixnum, brief_otherimm, brief_fixnum, brief_struct,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_list,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_otherptr,
-           brief_fixnum, brief_otherimm, brief_fixnum, brief_otherptr};
-#else
-    static void (*verbose_fns[])(lispobj obj)
-        = {print_fixnum, print_struct, print_otherimm, print_list,
-           print_fixnum, print_otherptr, print_otherimm, print_otherptr};
-    static void (*brief_fns[])(lispobj obj)
-        = {brief_fixnum, brief_struct, brief_otherimm, brief_list,
-           brief_fixnum, brief_otherptr, brief_otherimm, brief_otherptr};
-#endif
+#include "genesis/print.inc"
     int type = lowtag_of(obj);
     struct var *var = lookup_by_obj(obj);
     char buffer[256];
@@ -852,7 +854,7 @@ static void print_obj(char *prefix, lispobj obj)
         printf("%s0x%08lx: ", prefix, (unsigned long) obj);
         if (cur_depth < brief_depth) {
             fputs(lowtag_names[type], stdout);
-            fns = verbose_fns;
+            fns = print_fns;
         }
         else
             fns = brief_fns;
@@ -953,11 +955,11 @@ void safely_show_lstring(struct vector * string, int quotes, FILE *s)
       string = (struct vector*)forwarding_pointer_value((lispobj*)string);
   if (
 #ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-      widetag_of(string->header) == SIMPLE_CHARACTER_STRING_WIDETAG ||
+      header_widetag(string->header) == SIMPLE_CHARACTER_STRING_WIDETAG ||
 #endif
-      widetag_of(string->header) == SIMPLE_BASE_STRING_WIDETAG)
+      header_widetag(string->header) == SIMPLE_BASE_STRING_WIDETAG)
     show_lstring(string, quotes, s);
   else {
-    fprintf(s, "#<[widetag=%02X]>", widetag_of(string->header));
+    fprintf(s, "#<[widetag=%02X]>", header_widetag(string->header));
   }
 }
