@@ -295,14 +295,29 @@ static inline int simple_vector_p(lispobj obj) {
            widetag_of((lispobj*)(obj-OTHER_POINTER_LOWTAG)) == SIMPLE_VECTOR_WIDETAG;
 }
 
+/* This is NOT the same value that lisp's %INSTANCE-LENGTH returns.
+ * Lisp always uses the logical length (as originally allocated),
+ * except when heap-walking which requires exact physical sizes */
 static inline int instance_length(lispobj header)
 {
-  return HeaderValue(header) & SHORT_HEADER_MAX_WORDS;
+    // * Byte 3 of an instance header word holds the immobile gen# and visited bit,
+    //   so those have to be masked off.
+    // * fullcgc uses bit index 31 as a mark bit, so that has to
+    //   be cleared. Lisp does not have to clear bit 31 because fullcgc does not
+    //   operate concurrently.
+    // * If the object is in hashed-and-moved state and the original instance payload
+    //   length was odd (total object length was even), then add 1.
+    //   This can be detected by ANDing some bits, bit 10 being the least-significant
+    //   bit of the original size, and bit 9 being the 'hashed+moved' bit.
+    // * 64-bit machines do not need 'long' right-shifts, so truncate to int.
+
+    int extra = ((unsigned int)header >> 10) & ((unsigned int)header >> 9) & 1;
+    return (((unsigned int)header >> INSTANCE_LENGTH_SHIFT) & 0x3FFF) + extra;
 }
 
-/* Define an assignable instance_layout() macro taking a native pointer */
+/// instance_layout() macro takes a lispobj* and is an lvalue
 #ifndef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-# define instance_layout(instance_ptr) (instance_ptr)[1]
+# define instance_layout(instance_ptr) ((lispobj*)instance_ptr)[1]
 #elif defined(LISP_FEATURE_64_BIT) && defined(LISP_FEATURE_LITTLE_ENDIAN)
   // so that this stays an lvalue, it can't be cast to lispobj
 # define instance_layout(instance_ptr) ((uint32_t*)(instance_ptr))[1]
@@ -315,13 +330,25 @@ static inline int instance_length(lispobj header)
 static inline int
 is_lisp_pointer(lispobj obj)
 {
-#if N_WORD_BITS == 64
+#ifdef LISP_FEATURE_PPC64
+    return (obj & 5) == 4;
+#elif N_WORD_BITS == 64
     return (obj & 3) == 3;
 #else
     return obj & 1;
 #endif
 }
 
+/* The standard pointer tagging scheme admits an optimization that cuts the number
+ * of instructions down when testing for either 'a' or 'b' (or both) being a tagged
+ * pointer, because the bitwise OR of two pointers is considered a pointer.
+ * This trick is inadmissible for the PPC64 lowtag arrangement */
+#ifdef LISP_FEATURE_PPC64
+#define at_least_one_pointer_p(a,b) (is_lisp_pointer(a) || is_lisp_pointer(b))
+#else
+#define at_least_one_pointer_p(a,b) (is_lisp_pointer(a|b))
+#endif
+
 #include "fixnump.h"
 
 /* Is the Lisp object obj something with immediate nature (e.g. a
@@ -367,6 +394,8 @@ fixnum_value(lispobj n)
     return (sword_t)n >> N_FIXNUM_TAG_BITS;
 }
 
+lispobj symbol_function(lispobj* symbol);
+
 #include "align.h"
 
 #if defined(LISP_FEATURE_WIN32)
@@ -511,9 +540,8 @@ extern struct lisp_startup_options lisp_startup_options;
 #define BOXED_NWORDS(obj) ((HeaderValue(obj) & 0x7FFFFF) | 1)
 
 /* Medium-sized payload count is expressed in 15 bits. Objects in this category
- * may reside in immobile space: CLOSURE, INSTANCE, FUNCALLABLE-INSTANCE.
- * The single data bit is used as a closure's NAMED flag,
- * or an instance's "special GC strategy" flag.
+ * may reside in immobile space: CLOSURE, FUNCALLABLE-INSTANCE.
+ * The single data bit is used as a closure's NAMED flag.
  *
  * Header:  gen# |  data |     size |    tag
  *         -----   -----    -------   ------
